from __future__ import annotations

import json
from pathlib import Path

from PyQt5.QtWidgets import (
    QFormLayout,
    QHBoxLayout,
    QLabel,
    QLineEdit,
    QPushButton,
    QVBoxLayout,
    QWidget,
)


class SetupWindow(QWidget):
    def __init__(self, parent=None):
        super(QWidget, self).__init__()
        self.parent = parent

        layout = QVBoxLayout()
        form_layout = QFormLayout()
        button_box = QHBoxLayout()

        label_text = QLabel()
        label_text.setText('<center><font size="6">Initial Setup</font></center>')

        label_localdb_protocol = QLabel()
        label_localdb_protocol.setText("LocalDB Protocol (http or https):")

        self.edit_localdb_protocol = QLineEdit()
        self.edit_localdb_protocol.setText("http")

        label_localdb_host = QLabel()
        label_localdb_host.setText("LocalDB Hostname:")

        self.edit_localdb_host = QLineEdit()
        self.edit_localdb_host.setText("127.0.0.1")

        label_localdb_port = QLabel()
        label_localdb_port.setText("LocalDB Port:")

        self.edit_localdb_port = QLineEdit()
        self.edit_localdb_port.setText("5000")

        label_mongodb_host = QLabel()
        label_mongodb_host.setText("MongoDB Hostname:")

        self.edit_mongodb_host = QLineEdit()
        self.edit_mongodb_host.setText("127.0.0.1")

        label_mongodb_port = QLabel()
        label_mongodb_port.setText("MongoDB Port:")

        self.edit_mongodb_port = QLineEdit()
        self.edit_mongodb_port.setText("27017")

        Next_button = QPushButton("&Next")
        Next_button.clicked.connect(self.register_setup)

        button_box.addStretch()
        button_box.addWidget(Next_button)

        form_layout.addRow(label_localdb_protocol, self.edit_localdb_protocol)
        form_layout.addRow(label_localdb_host, self.edit_localdb_host)
        form_layout.addRow(label_localdb_port, self.edit_localdb_port)
        form_layout.addRow(label_mongodb_host, self.edit_mongodb_host)
        form_layout.addRow(label_mongodb_port, self.edit_mongodb_port)

        layout.addStretch()
        layout.addWidget(label_text)
        layout.addStretch()
        layout.addLayout(form_layout)
        layout.addLayout(button_box)

        self.setLayout(layout)

    def register_setup(self):
        with Path(
            self.parent.module_qc_nonelec_gui_dir + "/configuration/custom.json"
        ).open(mode="w", encoding="utf-8") as f:
            j = {
                "mongoDB": {
                    "address": self.edit_mongodb_host.text(),
                    "port": self.edit_mongodb_port.text(),
                },
                "localDB_web": {
                    "protocol": self.edit_localdb_protocol.text(),
                    "address": self.edit_localdb_host.text(),
                    "port": self.edit_localdb_port.text(),
                },
            }

            json.dump(j, f, indent=4)

        self.close()
        self.parent.init_ui()



from __future__ import annotations

import logging

from PyQt5.QtWidgets import QMainWindow

from module_qc_nonelec_gui.qc_tests.MASS_MEASUREMENT import user_input

log = logging.getLogger(__name__)


class TestWindow(QMainWindow):
    ############################################################################################
    def __init__(self, parent=None):
        super(QMainWindow, self).__init__()
        self.parent = parent
        self.initial_wid = None

        self.setWindowTitle("Mass Measurement")
        self.scale_window(340, 255)

        self.result_info = {
            "Scale_accuracy_value": "",
            "Scale_accuracy_unit": "mg",
            "mass_value": "",
            "mass_unit": "mg",
            "comment": "",
        }

    ###########################################################################################

    def init_ui(self):
        self.initial_wid = user_input.InitialWindow(self)
        self.parent.update_widget(self.initial_wid)

    def scale_window(self, x, y):
        self.setGeometry(0, 0, x, y)

    def close_and_return(self):
        self.close()
        self.parent.back_from_test()

    def back_page(self):
        self.parent.init_ui()

    def back_window(self):
        self.parent.receive_backpage()

    def call_another_window(self, window):
        self.hide()
        window.init_ui()

    def receive_mass(self, mass_value, acu_value, comment):
        # Essentially only here is the hard-coded part (value mapping ).
        self.parent.testRun["results"]["MASS"] = float(mass_value)
        self.parent.testRun["results"]["property"]["SCALE_ACCURACY"] = float(acu_value)
        self.parent.testRun["results"]["comment"] = str(comment)
        self.parent.testRun["results"]["metadata"]["MASS_UNIT"] = self.result_info[
            "mass_unit"
        ]
        self.parent.testRun["results"]["metadata"][
            "SCALE_ACCURACY_UNIT"
        ] = self.result_info["Scale_accuracy_unit"]

        self.parent.receive_result(self)



from __future__ import annotations

from pathlib import Path

from PyQt5.QtCore import Qt
from PyQt5.QtGui import QDoubleValidator, QIntValidator
from PyQt5.QtWidgets import (
    QFileDialog,
    QGridLayout,
    QHBoxLayout,
    QLabel,
    QLineEdit,
    QMessageBox,
    QPushButton,
    QScrollArea,
    QTextEdit,
    QVBoxLayout,
    QWidget,
)


class InitialWindow(QWidget):
    def __init__(self, parent=None):
        super(QWidget, self).__init__(parent)
        self.parent = parent

        layout = QVBoxLayout()
        bottom_box = QHBoxLayout()
        grid_edit = QGridLayout()

        label_title = QLabel()
        label_title.setText(
            '<center><font size="7">Input thermal cycling information</font></center>'
        )

        Next_button = QPushButton("&Next")
        Next_button.clicked.connect(self.pass_result)
        Back_button = QPushButton("&Back")
        Back_button.clicked.connect(self.back_page)

        bottom_box.addWidget(Back_button)
        bottom_box.addStretch()
        bottom_box.addWidget(Next_button)

        label_comment = QLabel()
        label_comment.setText("comment : ")
        label_Ncycle = QLabel()
        label_Ncycle.setText("# of Thermal Cycle  : ")
        label_cycle_speed = QLabel()
        label_cycle_speed.setText("Thermal Cycling Speed : ")
        label_Temp_min = QLabel()
        label_Temp_min.setText("Minimum Temperature : ")
        label_Temp_max = QLabel()
        label_Temp_max.setText("Maximum Temperature : ")

        self.edit_comment = QTextEdit()
        self.edit_comment.setText(self.parent.result_info["comment"])

        self.edit_machine = self.make_textform(
            "Machine : ", self.parent.result_info["Machine"], grid_edit, 0
        )
        #        self.edit_Ncycle   = self.make_textform('Number of cycle : ',self.parent.result_info['N_cycle'],grid_edit,1)

        grid_edit.addWidget(label_Ncycle, 1, 0)
        grid_edit.addLayout(self.make_Ncycle_layout(), 1, 1)

        grid_edit.addWidget(label_cycle_speed, 2, 0)
        grid_edit.addLayout(self.make_cycle_speed_layout(), 2, 1)

        grid_edit.addWidget(label_Temp_min, 3, 0)
        grid_edit.addLayout(self.make_Temp_min_layout(), 3, 1)

        grid_edit.addWidget(label_Temp_max, 4, 0)
        grid_edit.addLayout(self.make_Temp_max_layout(), 4, 1)

        self.temp_log = self.select_file_layout(
            "Temperature log : ", self.parent.result_info["Temp_log_path"], grid_edit, 5
        )
        self.humi_log = self.select_file_layout(
            "Humidity log : ", self.parent.result_info["Temp_log_path"], grid_edit, 6
        )

        grid_edit.addWidget(label_comment, 7, 0)

        outer = QScrollArea()
        outer.setWidgetResizable(True)
        outer.setWidget(self.edit_comment)
        grid_edit.addWidget(outer, 7, 1)

        layout.addWidget(label_title)
        layout.addStretch()
        layout.addLayout(grid_edit)
        layout.addLayout(bottom_box)

        self.setLayout(layout)

    def make_Temp_min_layout(self):
        grid_layout, self.edit_Temp_min = self.make_value_unit_layout(
            str(self.parent.result_info["Temp_min_value"]),
            self.parent.result_info["Temp_unit"],
        )

        return grid_layout

    def make_Temp_max_layout(self):
        grid_layout, self.edit_Temp_max = self.make_value_unit_layout(
            str(self.parent.result_info["Temp_max_value"]),
            self.parent.result_info["Temp_unit"],
        )

        return grid_layout

    def make_Ncycle_layout(self):
        #        grid_layout, self.edit_Ncycle =  self.make_value_unit_layout( str(self.parent.result_info['N_cycle']), '' )

        self.edit_Ncycle = QLineEdit()
        self.edit_Ncycle.setValidator(QIntValidator())
        self.edit_Ncycle.setAlignment(Qt.AlignRight)
        self.edit_Ncycle.setText(str(self.parent.result_info["N_cycle"]))

        layout = QHBoxLayout()
        layout.addWidget(self.edit_Ncycle)

        return layout

    def make_cycle_speed_layout(self):
        grid_layout, self.edit_cycle_speed = self.make_value_unit_layout(
            str(self.parent.result_info["Cycle_speed_value"]),
            self.parent.result_info["Cycle_speed_unit"],
        )

        return grid_layout

    def make_value_unit_layout(self, value_str, unit_str):
        grid_layout = QGridLayout()

        label = QLabel()
        label.setText(unit_str)

        edit_value = QLineEdit()
        edit_value.setValidator(QDoubleValidator())
        edit_value.setAlignment(Qt.AlignRight)
        edit_value.setText(value_str)

        grid_layout.addWidget(edit_value, 0, 0)
        grid_layout.addWidget(label, 0, 1)

        return grid_layout, edit_value

    def make_textform(self, label, initial, layout, i):
        label_text = QLabel()
        label_text.setText(label)
        edit_text = QLineEdit()
        edit_text.setText(initial)

        if initial == "TBA":
            edit_text.setStyleSheet("color: black; background-color:linen;")
            edit_text.setReadOnly(True)
            edit_text.setFocusPolicy(Qt.NoFocus)

        layout.addWidget(label_text, i, 0)
        layout.addWidget(edit_text, i, 1)

        return edit_text

    def select_file_layout(self, label, initial, layout, i):
        label_text = QLabel()
        label_text.setText(label)
        edit_text = QLineEdit()
        edit_text.setText(initial)
        edit_text.setFocusPolicy(Qt.NoFocus)
        edit_text.setReadOnly(True)

        Choose_button = QPushButton("Choose file")
        Choose_button.clicked.connect(lambda: self.get_path(edit_text))

        box = QHBoxLayout()
        box.addWidget(edit_text)
        box.addWidget(Choose_button)

        layout.addWidget(label_text, i, 0)
        layout.addLayout(box, i, 1)

        return edit_text

    def get_path(self, lineedit):
        lineedit.setText(self.choose_file())

    def choose_file(self):
        dlg = QFileDialog()
        dlg.setFileMode(QFileDialog.AnyFile)

        try:
            dlg.exec_()
            return dlg.selectedFiles()[0]
        except Exception:
            QMessageBox.warning(
                None, "Warning", "Please choose correct file", QMessageBox.Ok
            )
        return None

    def pass_result(self):
        token = 0
        try:
            result_dict = {
                "comment": self.edit_comment.toPlainText(),
                "Machine": self.edit_machine.text(),
                "Temp_log_path": self.temp_log.text(),
                "Humi_log_path": self.humi_log.text(),
                "Temp_min_value": float(self.edit_Temp_min.text()),
                "Temp_max_value": float(self.edit_Temp_max.text()),
                "N_cycle": int(self.edit_Ncycle.text()),
                "Cycle_speed_value": float(self.edit_cycle_speed.text()),
            }
            token = 1
        except Exception:
            #            import traceback
            #            print (traceback.format_exc() )
            QMessageBox.warning(
                None, "Warning", "Please fill these form.", QMessageBox.Ok
            )

        if token == 1:
            if int(self.edit_Ncycle.text()) < 0:
                QMessageBox.warning(
                    None,
                    "Warning",
                    "Negative value is not supported for the number of cycles.",
                    QMessageBox.Ok,
                )
            else:
                if (
                    Path(self.temp_log.text()).is_file()
                    and Path(self.humi_log.text()).is_file()
                ):
                    self.parent.receive_result(result_dict)
                else:
                    QMessageBox.critical(
                        None, "Warning", "Log file not exist.", QMessageBox.Ok
                    )

    def back_page(self):
        self.parent.close_and_return()



from __future__ import annotations

import json
import logging
from pathlib import Path

from PyQt5.QtWidgets import QMainWindow

from module_qc_nonelec_gui.qc_tests.FLATNESS import user_input

log = logging.getLogger(__name__)


class TestWindow(QMainWindow):
    ############################################################################################
    def __init__(self, parent=None):
        #        super(QMainWindow, self).__init__(parent)
        super(QMainWindow, self).__init__()
        self.parent = parent

        self.setGeometry(0, 0, 510, 255)

        self.result_info = {
            "backside_flatness": "",
            "angle_alpha": "",
            "angle_beta": "",
            "comment": "",
            "filename": "",
        }

        self.componentType = "MODULE"
        self.stage = "MODULE"

        self.init_ui()

    def receive_result(self, comment):
        with Path(self.result_info["filename"]).open(encoding="utf-8") as f:
            result_dict = json.load(f)

        self.parent.testRun["results"]["Metadata"] = result_dict

        for param in ["BACKSIDE_FLATNESS", "ANGLES"]:
            self.parent.testRun["results"][param] = result_dict[param]

        self.parent.testRun["results"]["comment"] = comment

        self.parent.receive_result(self)

    ############################################################################################
    def init_ui(self):
        self.user_input_wid = user_input.InitialWindow(self)
        self.parent.update_widget(self.user_input_wid)

    def close_and_return(self):
        self.close()
        self.parent.back_from_test()

    def back_page(self):
        self.parent.init_ui()

    def back_window(self):
        self.parent.receive_backpage()

    def call_another_window(self, window):
        self.hide()
        window.init_ui()



from __future__ import annotations

import logging

import itkdb

log = logging.getLogger(__name__)

##############
## functions


def upload_results(code1, code2, mod_name, stage, run_num, date, result1, result2):
    u = itkdb.core.User(access_code1=code1, access_code2=code2)
    pd_client = itkdb.Client(user=u)
    # queries = [{"status": "ready"}, {"status": "unmatched"}]
    pdtest_list = ["OPTICAL"]
    module_name = mod_name

    res1 = result1 == "1"
    res2 = result2 == "1"

    ## get the parent information and setting upload parameter
    parent_doc = pd_client.get("getComponent", json={"component": module_name})
    log.info("Module name: " + module_name)
    log.info("Stage:       " + stage)
    if parent_doc["currentStage"]["code"] != stage:
        log.info("Stage is not corresponded to ITk production DB. Please check it")
        # userdb.itkpd.uploader.commit.update_one({"_id":ObjectId(str(commit["_id"]))},{"set":{"status":"unmatched"}} )
    elif parent_doc["currentStage"]["code"] == stage:
        log.info("Stage is matched. Start to uploading test results...\n")
        log.info("There are " + str(len(pdtest_list)) + " test items to upload.")

        #  userdb.itkpd.uploader.commit.update_one({"_id":ObjectId(str(commit["_id"]))},{"set":{"status":"uploading"}} )
        for _i, testType in enumerate(pdtest_list):
            pd_testType = {"code": testType}

            child_doc = pd_client.get("getComponent", json={"component": module_name})
            project = {"project": child_doc["project"]["code"]}
            # subproject = {"subproject": child_doc["project"]["code"]}
            institution = {"institution": child_doc["currentLocation"]["code"]}
            componentType = {"componentType": child_doc["componentType"]["code"]}

            ## make the result page for parent component
            test_template = pd_client.get(
                "generateTestTypeDtoSample",
                json={**project, **componentType, **pd_testType},
            )
            #  new_test_result = pd_client.post('uploadTestRunResults', json={**test_template, 'component': module_name, **institution, 'runNumber':'0000', 'date':'23.04.2020', 'results':{'SCRATCHES':True,'DIRT':True}})
            pd_client.post(
                "uploadTestRunResults",
                json={
                    **test_template,
                    "component": module_name,
                    **institution,
                    "runNumber": run_num,
                    "date": date,
                    "results": {"SCRATCHES": res1, "DIRT": res2},
                },
            )
            log.info("Finished for all results!!\n")



from __future__ import annotations

import argparse
import sys
from logging import (
    INFO,
    getLogger,
)
from pathlib import Path

import yaml
from pymongo import MongoClient

logger = getLogger("Log")
logger.setLevel(INFO)


##################
def readConfig(conf_path):
    logger.debug("Read Config File.")
    if Path(conf_path).is_file():
        with Path(conf_path).open(encoding="utf-8") as f:
            return yaml.load(f, Loader=yaml.SafeLoader)
    return None


def readKey(i_path):
    with Path(i_path).open(encoding="utf-8") as file_text:
        file_keys = file_text.read().split()
        return {"username": file_keys[0], "password": file_keys[1]}


##################
# get arguments
def getArgs():
    logger.debug("Get Arguments.")
    parser = argparse.ArgumentParser(formatter_class=argparse.RawTextHelpFormatter)
    parser.add_argument(
        "command",
        help="option*\tfunction\ninit\tFunction initialization & Connection check",
        type=str,
        nargs="+",
    )
    parser.add_argument(
        "--config", help="Set User Config Path to Local DB Server.", type=str
    )
    parser.add_argument(
        "--username", help="Set the User Name of Local DB Server", type=str
    )
    parser.add_argument(
        "--password", help="Set the Password of Local DB Server", type=str
    )
    parser.add_argument("--database", help="Set Database Config Path", type=str)
    parser.add_argument("--user", help="Set User Config Path", type=str)
    parser.add_argument("--site", help="Set Site Config Path", type=str)
    parser.add_argument("--log", help="Set Log Mode", action="store_true")

    args = parser.parse_args()

    if args.config:
        conf = readConfig(args.config)
        if "username" in conf and not args.username:
            args.username = conf["username"]
        if "password" in conf and not args.password:
            args.password = conf["password"]

    return args


def init(db_cfg):
    logger.debug("Initialize.")

    if db_cfg == {}:
        logger.error("There is no default database config")
        sys.exit(1)

    url = f"mongodb://{db_cfg['hostIp']}:{db_cfg['hostPort']}"
    msg = f"[Connection Test] DB Server: {url}/{db_cfg.get('dbName', 'localdb')}"
    logger.info(msg)

    ##################
    # Connection check
    ##################

    # max_server_delay = 1
    # username = None
    # password = None
    # authSource = db_cfg.get("dbName", "localdb")
    ### check tls/ssl
    db_tls = db_cfg.get("tls", {}).get("enabled", False)
    db_ssl = db_cfg.get("ssl", {}).get("enabled", False)
    if db_tls:
        db_certfile = db_cfg.get("tls", {}).get("CertificateKeyFile", None)
        db_ca_certs = db_cfg.get("tls", {}).get("CAFile", None)
    elif db_ssl:
        db_certfile = db_cfg.get("ssl", {}).get("PEMKeyFile", None)
        db_ca_certs = db_cfg.get("ssl", {}).get("CAFile", None)
    if db_tls or db_ssl:
        url += f"/?ssl=true&ssl_ca_certs={db_ca_certs}&ssl_certfile={db_certfile}&ssl_match_hostname=false"
        ### authenticate mechanism
        if db_cfg.get("auth", None) == "x509":
            url += "&authMechnism=MONGODB-X509"
    # client = MongoClient(url, serverSelectionTimeoutMS=max_server_delay)
    # localdb = client[db_cfg.get("dbName", "localdb")]
    # try:
    #    # localdb['fs.files'].create_index([('hash', DESCENDING), ('_id', DESCENDING)])
    #    # lcoaldb['component'].create_index([('serialNumber', DESCENDING)])
    #    # localdb['testRun'].create_index([('startTime', DESCENDING),('user_id', DESCENDING), ('address', DESCENDING)])
    #    # localdb['componentTestRun'].create_index([('name', DESCENDING), ('testRun', DESCENDING)])
    #    # localdb["vi_results"].create_index([("component_id", DESCENDING)])
    # except errors.ServerSelectionTimeoutError as err:
    #     ### Connection failed
    #     logger.error("---> Connection is BAD.")
    #     logger.error(f"     {err}")
    #     logger.error(
    #         "     Access https://localdb-docs.readthedocs.io/en/master/faq/ to check more detail"
    #     )
    # except errors.OperationFailure as err:
    #     ### Need user authentication
    #     if db_cfg.get("KeyFile", None) and db_cfg["KeyFile"] != "null":
    #         keys = readKey(db_cfg["KeyFile"])
    #         username = keys["username"]
    #         password = keys["password"]
    #     if args.username:
    #         username = args.username
    #     elif db_cfg.get("username", None):
    #         username = db_cfg["username"]
    #     elif os.environ.get("username", None):
    #         username = os.environ["username"]
    #     if args.password:
    #         password = args.password
    #     elif db_cfg.get("password", None):
    #         password = db_cfg.get("password", None)
    #     elif os.environ.get("password", None):
    #         password = os.environ["password"]
    #     if username and password:
    #         try:
    #             localdb.authenticate(username, password)
    #         except errors.OperationFailure as err:
    #             logger.error("Authentication failed.")
    #             return False
    #     else:
    #         return False

    #     # localdb["vi_results"].create_index([("component_id", DESCENDING)])

    #     # register.__set_localdb(localdb)
    #     # register.__set_list(db_cfg.get("stage", []), "stage")
    #     # register.__set_list(db_cfg.get("environment", []), "environment")
    #     # register.__set_list(db_cfg.get("component", []), "component")

    #     logger.info("---> Connection is GOOD.")

    return True


def connectDB():
    return MongoClient(port=27017)


def write_vi_result(moduleid, inspector, stage, results, comments):
    c = connectDB()
    localdb = c["test"]

    localdb.vi_collection.insert_one(
        {"compomet_name": moduleid},
        {"inspector": inspector},
        {"stage": stage},
        {"result": results},
        {"comment": comments},
    )
    # localdb.test_collection.insert_one({"component_name":'20UPBMO0000007',"stage":'TEST',"number":1 })


def update_vi_result(moduleid, results, comments):
    c = connectDB()
    localdb = c["test"]

    localdb.vi_collection.update_one(
        {"compomet_name": moduleid}, {set: {"result": results}}
    )
    localdb.vi_collection.update_one(
        {"compomet_name": moduleid}, {set: {"comment": comments}}
    )
    # localdb.test_collection.insert_one({"component_name":'20UPBMO0000007',"stage":'TEST',"number":1 })



from __future__ import annotations

from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import (
    QFormLayout,
    QHBoxLayout,
    QLabel,
    QLineEdit,
    QPushButton,
    QScrollArea,
    QTextEdit,
    QVBoxLayout,
    QWidget,
)


class NotSupportedError(Exception):
    pass


class ConfirmWindow(QWidget):
    def __init__(self, parent=None):
        super(QWidget, self).__init__(parent)
        self.parent = parent

        titlebox = QVBoxLayout()
        layout = QVBoxLayout()
        button_box = QHBoxLayout()

        label_title = QLabel()
        label_title.setText(
            '<center><font size="5">Confirm before uploading to the database</font></center>'
        )
        label_practice = QLabel()
        label_practice.setText(
            '<center><font size="4" color = "green"> Practice Mode</font></center>'
        )
        Upload_button = QPushButton("&Upload!")
        Upload_button.clicked.connect(self.upload_to_db)
        json_button = QPushButton("&Check json (for expert)")
        json_button.clicked.connect(self.check_json)
        back_button = QPushButton("&Back")
        back_button.clicked.connect(self.back_page)

        titlebox.addWidget(label_title)
        if self.parent.isPractice:
            titlebox.addWidget(label_practice)

        button_box.addWidget(back_button)
        button_box.addStretch()
        button_box.addWidget(json_button)
        button_box.addWidget(Upload_button)

        inner = QScrollArea()
        inner.setFixedHeight(400)
        inner.setFixedWidth(600)
        result_wid = QWidget()
        result_wid.setLayout(self.make_layout())

        inner.setWidgetResizable(True)
        inner.setWidget(result_wid)

        layout.addLayout(titlebox)
        layout.addWidget(inner)
        layout.addLayout(button_box)
        self.setLayout(layout)

    def back_page(self):
        self.parent.back_to_test()

    def check_json(self):
        self.parent.confirm_json()

    def upload_to_db(self):
        self.parent.upload_to_db()

    def add_info(self, Form_layout, label_str, form_text):
        label = QLabel()
        label.setText(label_str)

        if label_str == "Comment :":
            inner = QTextEdit()
            inner.setText(form_text)
            inner.setReadOnly(True)
            inner.setFocusPolicy(Qt.NoFocus)
            inner.setStyleSheet("background-color : linen; color: black;")

            editor = QScrollArea()
            editor.setWidgetResizable(True)
            editor.setWidget(inner)

        else:
            editor = QLineEdit()
            editor.setText(form_text)
            editor.setReadOnly(True)
            editor.setFocusPolicy(Qt.NoFocus)
            editor.setStyleSheet("background-color : linen; color: black;")
        #        editor.setStyleSheet("background-color : azure")

        Form_layout.addRow(label, editor)

    #################################################################
    def make_layout(self):
        if (
            self.parent.info_dict["componentType"] == "MODULE"
            or self.parent.info_dict["componentType"] == "practice"
        ):
            return self.layout_ModuleQC()
        return self.layout_ModuleQC()
        #            raise NotSupportedError('ComponentType "' + self.parent.info_dict['componentType'] +'" is not supported')

    def layout_ModuleQC(self):
        Form_layout = QFormLayout()
        self.add_info(
            Form_layout,
            "Current Stage :",
            self.parent.result_dict["localDB"]["currentStage"],
        )
        self.add_info(
            Form_layout, "Test Type :", self.parent.result_dict["localDB"]["testType"]
        )

        self.add_info(
            Form_layout,
            "Machine :",
            self.parent.result_dict["localDB"]["results"]["property"]["Machine"],
        )
        self.add_info(
            Form_layout,
            "# of Thermal Cycle :",
            str(self.parent.result_dict["localDB"]["results"]["property"]["N_cycle"]),
        )

        Cycle_speed = (
            str(
                self.parent.result_dict["localDB"]["results"]["property"][
                    "Cycle_speed_value"
                ]
            )
            + " "
            + self.parent.result_dict["localDB"]["results"]["property"][
                "Cycle_speed_unit"
            ]
        )
        Min_temp = (
            str(
                self.parent.result_dict["localDB"]["results"]["property"][
                    "Temp_min_value"
                ]
            )
            + " "
            + self.parent.result_dict["localDB"]["results"]["Temp_unit"]
        )
        Max_temp = (
            str(
                self.parent.result_dict["localDB"]["results"]["property"][
                    "Temp_max_value"
                ]
            )
            + " "
            + self.parent.result_dict["localDB"]["results"]["Temp_unit"]
        )

        self.add_info(Form_layout, "Thermal Cycling Speed :", Cycle_speed)
        self.add_info(Form_layout, "Minimum Temperature :", Min_temp)
        self.add_info(Form_layout, "Maximum Temperature :", Max_temp)

        self.add_info(
            Form_layout,
            "Temperature log :",
            self.parent.test_result_dict["results"]["summary"]["Temp_log_path"],
        )
        self.add_info(
            Form_layout,
            "Humidity log :",
            self.parent.test_result_dict["results"]["summary"]["Humi_log_path"],
        )

        self.add_info(
            Form_layout,
            "Comment :",
            self.parent.result_dict["localDB"]["results"]["comment"],
        )

        return Form_layout



from __future__ import annotations

import math

import cv2


def read_img(path):
    img = cv2.imread(path, 1)
    h, w, d = img.shape
    return img, h, w, d


def scale_img(img, w_scale, h_scale):
    img_scaled = cv2.resize(img, (int(w_scale), int(h_scale)))
    h, w, d = img_scaled.shape
    return img_scaled, h, w, d


def img_cvt_rgb(img):
    img_rgb = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
    h, w, d = img_rgb.shape
    return img_rgb, h, w, d


def img_rotate(img):
    img_rotated = cv2.rotate(img, cv2.ROTATE_90_CLOCKWISE)
    h, w, d = img_rotated.shape
    return img_rotated, h, w, d


def img_trim(img, y_min, y_max, x_min, x_max):
    return img[y_min:y_max, x_min:x_max]


def write_img(img, path):
    cv2.imwrite(path, img)
    return 0


def draw_circle(img, x, y, r):
    return cv2.circle(img, (x, y), r, (0, 0, 255), 10)


def find_circle(x, y, circles, min_dist):
    for circle in circles:
        if cal_distance(x, y, circle[0], circle[1]) < min_dist:
            c_x = circle[0]
            c_y = circle[1]
    return c_x, c_y


def cal_distance(x1, y1, x2, y2):
    return math.sqrt((x1 - x2) * (x1 - x2) + (y1 - y2) * (y1 - y2))


def rotate_img(img, h, w, dx, dy):
    theta = math.atan2(dy, dx)
    degree = math.degrees(theta)
    mat = cv2.getRotationMatrix2D((w / 2, h / 2), degree, 1)
    affine_img = cv2.warpAffine(img, mat, (w, h))

    return affine_img, mat



from __future__ import annotations

from pathlib import Path

from PyQt5.QtCore import Qt
from PyQt5.QtGui import QDoubleValidator
from PyQt5.QtWidgets import (
    QComboBox,
    QFileDialog,
    QGridLayout,
    QHBoxLayout,
    QLabel,
    QLineEdit,
    QMessageBox,
    QPushButton,
    QScrollArea,
    QTextEdit,
    QVBoxLayout,
    QWidget,
)


class InitialWindow(QWidget):
    def __init__(self, parent=None):
        super(QWidget, self).__init__(parent)
        self.parent = parent

        layout = QVBoxLayout()
        bottom_box = QHBoxLayout()
        grid_edit = QGridLayout()

        label_title = QLabel()
        label_title.setText(
            '<center><font size="7">Input Wirebonding information</font></center>'
        )

        Next_button = QPushButton("&Next")
        Next_button.clicked.connect(self.pass_result)
        Back_button = QPushButton("&Back")
        Back_button.clicked.connect(self.back_page)

        bottom_box.addWidget(Back_button)
        bottom_box.addStretch()
        bottom_box.addWidget(Next_button)

        label_comment = QLabel()
        label_comment.setText("comment : ")
        label_roomTemp = QLabel()
        label_roomTemp.setText("Room temperature : ")
        label_humidity = QLabel()
        label_humidity.setText("Humidity : ")

        self.edit_comment = QTextEdit()
        self.edit_comment.setText(self.parent.result_info["comment"])

        self.edit_machine = self.make_textform(
            "Machine used : ", self.parent.result_info["Machine"], grid_edit, 0
        )
        self.edit_operator = self.make_textform(
            "Operator Name : ", self.parent.result_info["Operator"], grid_edit, 1
        )
        self.edit_institution = self.make_inscombo(
            "Institution of Operator : ",
            self.parent.result_info["Institution_of_operator"],
            grid_edit,
            2,
        )
        self.edit_batch = self.make_textform(
            "Bond wire batch : ", self.parent.result_info["batch"], grid_edit, 3
        )
        self.edit_jig = self.make_textform(
            "Bonding jig : ", self.parent.result_info["jig"], grid_edit, 4
        )

        self.edit_program = self.select_file_layout(
            "Bond program : ", self.parent.result_info["program_path"], grid_edit, 5
        )

        grid_edit.addWidget(label_roomTemp, 6, 0)
        grid_edit.addLayout(self.make_roomTemp_layout(), 6, 1)

        grid_edit.addWidget(label_humidity, 7, 0)
        grid_edit.addLayout(self.make_humidity_layout(), 7, 1)

        grid_edit.addWidget(label_comment, 8, 0)

        outer = QScrollArea()
        outer.setWidgetResizable(True)
        outer.setWidget(self.edit_comment)
        grid_edit.addWidget(outer, 8, 1)

        layout.addWidget(label_title)
        layout.addStretch()
        layout.addLayout(grid_edit)
        layout.addLayout(bottom_box)

        self.setLayout(layout)

    def make_inscombo(self, label, initial, layout, i):
        combo = self.make_combobox(label, initial, layout, i)
        combo.addItem("")
        combo.addItems(list(self.parent.result_info["Institutions"].keys()))

        combo.lineEdit().textEdited.connect(
            lambda: self.inscomobo_refresh(combo, combo.currentText())
        )
        return combo

    def inscomobo_refresh(self, combo, currenttext):
        combo.clear()
        combo.addItem("")
        combo.lineEdit().setText(currenttext)
        if currenttext != "":
            new_list = [
                element
                for element in list(self.parent.result_info["Institutions"].keys())
                if currenttext.upper() in element.upper()
            ]
            combo.addItems(new_list)
        else:
            combo.addItems(list(self.parent.result_info["Institutions"].keys()))

    def make_combobox(self, label, initial, layout, i):
        label_text = QLabel()
        label_text.setText(label)
        combo = QComboBox()
        LE = QLineEdit()
        combo.setLineEdit(LE)
        combo.lineEdit().setText(initial)
        #        combo.lineEdit().setCompleter(None)
        if initial == "TBA":
            LE.setReadOnly(True)
            #            combo.lineEdit().setReadOnly(True)
            combo.setStyleSheet("color: black; background-color:linen;")
            combo.setFocusPolicy(Qt.NoFocus)

        layout.addWidget(label_text, i, 0)
        layout.addWidget(combo, i, 1)

        return combo

    def make_roomTemp_layout(self):
        grid_layout, self.edit_roomTemp = self.make_value_unit_layout(
            str(self.parent.result_info["Temp_value"]),
            self.parent.result_info["Temp_unit"],
        )

        return grid_layout

    def make_humidity_layout(self):
        grid_layout, self.edit_humidity = self.make_value_unit_layout(
            str(self.parent.result_info["Humidity_value"]),
            self.parent.result_info["Humidity_unit"],
        )

        return grid_layout

    def make_value_unit_layout(self, value_str, unit_str):
        grid_layout = QGridLayout()

        label = QLabel()
        label.setText(unit_str)

        edit_value = QLineEdit()
        edit_value.setValidator(QDoubleValidator())
        edit_value.setAlignment(Qt.AlignRight)
        edit_value.setText(value_str)

        grid_layout.addWidget(edit_value, 0, 0)
        grid_layout.addWidget(label, 0, 1)

        return grid_layout, edit_value

    def make_textform(self, label, initial, layout, i):
        label_text = QLabel()
        label_text.setText(label)
        edit_text = QLineEdit()
        edit_text.setText(initial)

        if initial == "TBA":
            edit_text.setStyleSheet("color: black; background-color:linen;")
            edit_text.setReadOnly(True)
            edit_text.setFocusPolicy(Qt.NoFocus)

        layout.addWidget(label_text, i, 0)
        layout.addWidget(edit_text, i, 1)

        return edit_text

    def select_file_layout(self, label, initial, layout, i):
        label_text = QLabel()
        label_text.setText(label)
        edit_text = QLineEdit()
        edit_text.setText(initial)
        edit_text.setFocusPolicy(Qt.NoFocus)
        edit_text.setReadOnly(True)

        Choose_button = QPushButton("Choose file")
        Choose_button.clicked.connect(lambda: self.get_path(edit_text))

        box = QHBoxLayout()
        box.addWidget(edit_text)
        box.addWidget(Choose_button)

        layout.addWidget(label_text, i, 0)
        layout.addLayout(box, i, 1)

        return edit_text

    def get_path(self, lineedit):
        lineedit.setText(self.choose_file())

    def choose_file(self):
        dlg = QFileDialog()
        dlg.setFileMode(QFileDialog.AnyFile)

        try:
            dlg.exec_()
            return dlg.selectedFiles()[0]
        except Exception:
            QMessageBox.warning(
                None, "Warning", "Please choose correct file", QMessageBox.Ok
            )
        return None

    def pass_result(self):
        token = 0
        try:
            ins_token = 0
            if {self.edit_institution.currentText()} <= set(
                self.parent.result_info["Institutions"].keys()
            ):
                ins_token = 1
            else:
                ins_msgBox = QMessageBox.warning(
                    None,
                    "Warning",
                    "The institution is not registered in ITkPD. Do you want to continue?",
                    QMessageBox.Yes | QMessageBox.No,
                )
                if ins_msgBox == QMessageBox.Yes:
                    ins_token = 1
                elif ins_msgBox == QMessageBox.No:
                    ins_token = 0
            if ins_token == 1:
                result_dict = {
                    "comment": self.edit_comment.toPlainText(),
                    "Machine": self.edit_machine.text(),
                    "Operator": self.edit_operator.text(),
                    "Institution_of_operator": self.edit_institution.currentText(),
                    "batch": self.edit_batch.text(),
                    "jig": self.edit_jig.text(),
                    "program_path": self.edit_program.text(),
                    "Temp_value": float(self.edit_roomTemp.text()),
                    "Humidity_value": float(self.edit_humidity.text()),
                }
                token = 1
        except Exception:
            QMessageBox.warning(
                None, "Warning", "Please fill these form.", QMessageBox.Ok
            )

        if token == 1:
            if (
                float(self.edit_roomTemp.text()) < -50
                or float(self.edit_roomTemp.text()) > 50
            ):
                QMessageBox.warning(
                    None, "Warning", "Temperature looks wrong.", QMessageBox.Ok
                )
            else:
                if (
                    float(self.edit_humidity.text()) < 0
                    or float(self.edit_humidity.text()) > 100
                ):
                    QMessageBox.warning(
                        None,
                        "Warning",
                        "It is impossible humidity value.",
                        QMessageBox.Ok,
                    )
                else:
                    if Path(self.edit_program.text()).is_file():
                        self.parent.receive_result(result_dict)
                    else:
                        QMessageBox.critical(
                            None, "Warning", "Program file not exist.", QMessageBox.Ok
                        )

    def back_page(self):
        self.parent.close_and_return()



from __future__ import annotations

import logging
import traceback

from PyQt5.QtCore import Qt
from PyQt5.QtGui import QDoubleValidator
from PyQt5.QtWidgets import (
    QGridLayout,
    QHBoxLayout,
    QLabel,
    QLineEdit,
    QMessageBox,
    QPushButton,
    QScrollArea,
    QTextEdit,
    QVBoxLayout,
    QWidget,
)

log = logging.getLogger(__name__)


class InitialWindow(QWidget):
    def __init__(self, parent=None):
        super(QWidget, self).__init__(parent)
        self.parent = parent

        layout = QVBoxLayout()
        bottom_box = QHBoxLayout()
        grid_edit = QGridLayout()

        label_title = QLabel()
        label_title.setText('<center><font size="7">Input mass value</font></center>')

        Next_button = QPushButton("&Next")
        Next_button.clicked.connect(self.pass_result)
        Back_button = QPushButton("&Back")
        Back_button.clicked.connect(self.back_page)

        bottom_box.addWidget(Back_button)
        bottom_box.addStretch()
        bottom_box.addWidget(Next_button)

        label_text = QLabel()
        label_text.setText("Total weight : ")
        label_acu = QLabel()
        label_acu.setText("Scale Accuracy : ")
        label_unit = QLabel()
        label_unit.setText(self.parent.result_info["mass_unit"])
        label_acu_unit = QLabel()
        label_acu_unit.setText(self.parent.result_info["Scale_accuracy_unit"])
        label_comment = QLabel()
        label_comment.setText("comment : ")

        self.edit_comment = QTextEdit()
        self.edit_comment.setText(self.parent.result_info["comment"])
        self.edit_mass = QLineEdit()
        self.edit_mass.setAlignment(Qt.AlignRight)
        self.edit_mass.setValidator(QDoubleValidator())
        self.edit_mass.setFixedWidth(100)
        self.edit_mass.setMaxLength(8)
        if self.parent.result_info["mass_value"] != "":
            self.edit_mass.setText(self.parent.result_info["mass_value"])
        else:
            self.edit_mass.setText("")

        self.edit_acu = QLineEdit()
        self.edit_acu.setAlignment(Qt.AlignRight)
        self.edit_acu.setValidator(QDoubleValidator())
        self.edit_acu.setFixedWidth(100)
        self.edit_acu.setMaxLength(8)
        if self.parent.result_info["Scale_accuracy_value"] != "":
            self.edit_mass.setText(self.parent.result_info["Scale_accuracy_value"])
        else:
            self.edit_mass.setText("")

        grid_edit.addWidget(label_text, 0, 0)
        grid_edit.addWidget(self.edit_mass, 0, 1, Qt.AlignRight)
        grid_edit.addWidget(label_unit, 0, 2)

        grid_edit.addWidget(label_acu, 1, 0)
        grid_edit.addWidget(self.edit_acu, 1, 1, Qt.AlignRight)
        grid_edit.addWidget(label_acu_unit, 1, 2)

        grid_edit.addWidget(label_comment, 2, 0)

        outer = QScrollArea()
        outer.setWidgetResizable(True)
        outer.setWidget(self.edit_comment)
        grid_edit.addWidget(outer, 2, 1)

        layout.addWidget(label_title)
        layout.addStretch()
        layout.addLayout(grid_edit)
        layout.addLayout(bottom_box)

        self.setLayout(layout)

    def pass_result(self):
        try:
            mass_value = self.edit_mass.text()
            acu_value = self.edit_acu.text()
            free_comment = self.edit_comment.toPlainText()
        except Exception:
            log.exception(traceback.format_exc())
            QMessageBox.warning(
                None, "Warning", "Please input target weight.", QMessageBox.Ok
            )

        if self.validator(mass_value) == 0:
            QMessageBox.warning(
                None, "Warning", "Please input target weight.", QMessageBox.Ok
            )
        if self.validator(acu_value) == 0:
            QMessageBox.warning(
                None, "Warning", "Please input scale accuracy.", QMessageBox.Ok
            )
        if (self.validator(mass_value) == -1) or (self.validator(acu_value) == -1):
            valid_msg = QMessageBox.warning(
                None, "Warning", "Negative value is not supported", QMessageBox.Ok
            )
        elif (self.validator(mass_value) == -2) or (self.validator(acu_value) == -2):
            valid_msg = QMessageBox.warning(
                None,
                "Warning",
                "your input looks too large. Do you want to continue?",
                QMessageBox.Yes | QMessageBox.No,
            )
            if valid_msg == QMessageBox.Yes:
                self.parent.receive_mass(mass_value, acu_value, free_comment)  # changed
            elif valid_msg == QMessageBox.No:
                pass
        elif (self.validator(mass_value) == 1) and (self.validator(acu_value) == 1):
            self.parent.receive_mass(mass_value, acu_value, free_comment)

    def back_page(self):
        self.parent.close_and_return()

    def validator(self, number_text):
        mode = 0

        try:
            float_number = float(number_text)
        except Exception:
            return mode

        if float_number < 0:
            mode = -1
        elif float_number > 10000:
            mode = -2
        else:
            mode = 1

        return mode



from __future__ import annotations

import json
import logging
from pathlib import Path

import gridfs

log = logging.getLogger(__name__)


def jpeg_formatter(localdb, path):
    fs = gridfs.GridFS(localdb)
    with Path(path).open(mode="rb") as f:
        binary = f.read()
    return fs.put(binary)


def uploader(localdb, json_file):
    # localdb = connectDB()
    try:
        with Path(json_file).open(encoding="utf-8") as f:
            json_load = json.load(f)
    except FileNotFoundError:
        json_load = json.loads(json_file)
    except Exception:
        json_load = json_file
    # if testname == "visual_inspection":
    localdb.QC.result.insert_one(json_load)


def property_uploader(localdb, json_file):
    # localdb = connectDB()
    try:
        with Path(json_file).open(encoding="utf-8") as f:
            json_load = json.load(f)
    except FileNotFoundError:
        json_load = json.loads(json_file)
    except Exception:
        json_load = json_file
    # if testname == "visual_inspection":
    localdb.QC.module.prop.insert_one(json_load)


def updater(localdb, testname, module_id, stage, path_jpegs, dt):
    # localdb = connectDB()
    img_dic = {}
    for page, path_jpeg in path_jpegs.items():
        img_dic[page] = str(jpeg_formatter(localdb, path_jpeg))

    if testname == "Optical":
        # module = localdb.component.find_one(
        #     {"and": [{"serialNumber": module_id}, {"componentType": "module"}]}
        # )
        doc = localdb.QC.result.find_one(
            {"and": [{"component": module_id}, {"stage": stage}, {"sys": {"mts": dt}}]}
        )
        log.info(doc)
        log.info(doc["results"])
        # # log.info(type(doc))
        # dic = doc["results"][0]
        # log.info(type(dic))
        anomaly_list = doc["results"]["anomaly"]
        comment_list = doc["results"]["comment"]
        localdb.QC.result.find_one_and_update(
            {"and": [{"component": module_id}, {"stage": stage}]},
            {
                "set": {
                    "results": {
                        "anomaly": anomaly_list,
                        "comment": comment_list,
                        "img": img_dic,
                    }
                }
            },
        )


def find_and_show(localdb, testname, _module_id, _stage):
    # localdb = connectDB()
    # doc = localdb.collectionname.find({'and':[{"component_name":module_id}, {"stage":stage}]})
    if testname == "Optical":
        docs = localdb.QC.result.find()
    for doc in docs:
        log.info(doc)



from __future__ import annotations


def module_retriever(localdb, serialNumber):
    return localdb.component.find_one({"and": [{"serialNumber": serialNumber}]})


def moduleid_retriever(localdb, serialNumber):
    try:
        module = localdb.component.find_one(
            {"and": [{"serialNumber": serialNumber}, {"componentType": "module"}]}
        )
        return str(module["_id"])
    except TypeError:
        try:
            module = localdb.component.find_one(
                {
                    "and": [
                        {"serialNumber": serialNumber},
                        {"componentType": "bare_module"},
                    ]
                }
            )
            return str(module["_id"])
        except TypeError:
            module = localdb.component.find_one(
                {
                    "and": [
                        {"serialNumber": serialNumber},
                        {"componentType": "module_pcb"},
                    ]
                }
            )
            return str(module["_id"])


def stage_retriever(localdb, serialNumber):
    mod_id = moduleid_retriever(localdb, serialNumber)
    qc_mod_status = localdb.QC.module.status.find_one({"component": mod_id})
    return qc_mod_status["currentStage"]


def userinfo_retriever(localdbtools, username):
    return localdbtools.viewer.user.find_one({"username": username})


def status_check(localdb, serialNumber, testname, isProperty):
    mod_id = moduleid_retriever(localdb, serialNumber)
    stage = stage_retriever(localdb, serialNumber)
    if isProperty:
        doc = localdb.QC.module.prop.find(
            {
                "and": [
                    {"component": mod_id},
                    {"currentStage": stage},
                    {"testType": testname},
                ]
            }
        )
    else:
        doc = localdb.QC.result.find(
            {
                "and": [
                    {"component": mod_id},
                    {"currentStage": stage},
                    {"testType": testname},
                ]
            }
        )
    ndoc = doc.count()
    user = doc.sort([("natural", -1)])[0]["user"]
    address = doc.sort([("natural", -1)])[0]["address"]
    date = doc.sort([("natural", -1)])[0]["sys"]["cts"]
    return {"ndoc": ndoc, "user": user, "address": address, "date": date}



from __future__ import annotations

from PyQt5.QtWidgets import (
    QGridLayout,
    QLabel,
    QLineEdit,
    QPushButton,
    QWidget,
)

# from PyQt5.QtWidgets import QGridLayout, QLabel, QLineEdit, QPushButton, QWidget
# from PyQt5.QtGUI import *
#


class InitialWindow(QWidget):
    def __init__(self, parent=None):
        super(QWidget, self).__init__(parent)
        self.parent = parent

        # Check if doing VI for PCB
        is_PCB = "PCB" in self.parent.type_name
        # Add instrument to the parent
        self.parent.instrument = ""

        # layout
        layout = QGridLayout()

        # QLabel
        label_text = QLabel()
        label_text.setText(
            f'<center><font size="6"> {self.parent.mode.capitalize()}-Side Inspection</font><center>'
        )
        label_atlsn = QLabel()
        label_atlsn.setText("ATLAS Serial Number:")
        label_typename = QLabel()
        label_typename.setText("Component Type:")
        # label_orginstitution = QLabel()
        # label_orginstitution.setText("Original Institution:")
        # label_currentlocation = QLabel()
        # label_currentlocation.setText("Current Location:")
        label_currentstage = QLabel()
        label_currentstage.setText("Current Stage:")
        label_inspector = QLabel()
        label_inspector.setText("Inspector:")
        label_institute = QLabel()
        label_institute.setText("Institution:")
        # Add one QLabel with instrument for PCB Visual Inspection
        if is_PCB:
            label_instrument = QLabel()
            label_instrument.setText("Instrument:")

        # QLineEdit
        self.edit_atlsn = QLineEdit()
        self.edit_typename = QLineEdit()
        # self.edit_origininstitution = QLineEdit()
        # self.edit_currentlocation = QLineEdit()
        self.edit_currentstage = QLineEdit()
        self.edit_inspector = QLineEdit()
        # Add one QLineEdit with instrument for PCB Visual Inspection
        if is_PCB:
            self.edit_instrument = QLineEdit()

        # Differentiate between PCB and others
        if is_PCB:
            for e in [
                self.edit_atlsn,
                self.edit_typename,
                self.edit_currentstage,
            ]:
                e.setReadOnly(True)
                e.setStyleSheet("background-color : linen; color: black;")
        else:
            for e in [
                self.edit_atlsn,
                self.edit_typename,
                self.edit_currentstage,
                self.edit_inspector,
            ]:
                e.setReadOnly(True)
                e.setStyleSheet("background-color : linen; color: black;")

        self.edit_atlsn.setText(self.parent.atlsn)
        self.edit_typename.setText(self.parent.type_name)
        # self.edit_origininstitution.setText(self.parent.original_institution)
        # self.edit_currentlocation.setText(self.parent.current_location)
        if self.parent.type_name == "MODULE":
            self.edit_currentstage.setText(self.parent.stage)
            self.edit_inspector.setText(self.parent.inspector)
            # self.edit_institute.setText()
        else:
            self.edit_currentstage.setText(self.parent.stage)

        # Button
        button_layout = QGridLayout()

        button = QPushButton("&Select Image...")

        button.clicked.connect(self.chooseimage)
        button_back = QPushButton("&Back")
        button_back.clicked.connect(self.backpage)

        button_layout.addWidget(button_back, 0, 0)
        button_layout.addWidget(button, 0, 2)

        # Set Layout
        layout.addWidget(label_text, 0, 0, 1, 2)
        layout.addWidget(label_atlsn, 1, 0)
        layout.addWidget(self.edit_atlsn, 1, 1)
        layout.addWidget(label_typename, 2, 0)
        layout.addWidget(self.edit_typename, 2, 1)
        # layout.addWidget(label_orginstitution, 3, 0)
        # layout.addWidget(self.edit_origininstitution, 3, 1)
        # layout.addWidget(label_currentlocation, 4, 0)
        # layout.addWidget(self.edit_currentlocation, 4, 1)
        layout.addWidget(label_currentstage, 5, 0)
        layout.addWidget(self.edit_currentstage, 5, 1)
        layout.addWidget(label_inspector, 6, 0)
        layout.addWidget(self.edit_inspector, 6, 1)
        # Add instrument line to layout for PCB
        if is_PCB:
            layout.addWidget(label_instrument, 7, 0)
            layout.addWidget(self.edit_instrument, 7, 1)
        # layout.addWidget(button,8,2)
        layout.addLayout(button_layout, 8, 0, 1, 2)

        layout.setSpacing(10)
        layout.setContentsMargins(20, 20, 20, 20)
        self.setLayout(layout)

    def chooseimage(self):
        # self.parent.atlsn = self.edit_atlsn.text()
        # self.parent.type_name = self.edit_typename.text()
        # self.parent.inspector = self.edit_inspector.text()
        # Need to read-out operator and instrument field
        if "PCB" in self.parent.type_name:
            self.parent.inspector = self.edit_inspector.text()
            self.parent.instrument = self.edit_instrument.text()

        self.parent.load_img(self.parent.mode)

    def backpage(self):
        if self.parent.mode == "front":
            self.parent.close_and_return()
        else:
            self.parent.mode = "front"
            self.parent.go_to_summary(self)



from __future__ import annotations

import json
import logging
import traceback
from pathlib import Path

import jsonschema
from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import (
    QFileDialog,
    QGridLayout,
    QHBoxLayout,
    QLabel,
    QLineEdit,
    QMessageBox,
    QPushButton,
    QScrollArea,
    QTextEdit,
    QVBoxLayout,
    QWidget,
)

from module_qc_nonelec_gui.qc_tests.IV_MEASURE_BARE import function

log = logging.getLogger(__name__)


class InitialWindow(QWidget):
    def __init__(self, parent=None):
        super(QWidget, self).__init__(parent)
        self.parent = parent
        ###############################################################
        ########         json schema file path                 ########
        try:
            self.jsonschema_path = Path(__file__).parent.joinpath(
                "example", "jsonschema", "validation_schema.json"
            )
        except Exception:
            self.jsonschema_path = ""
        ###############################################################

        layout = QVBoxLayout()
        bottom_box = QHBoxLayout()
        grid_edit = QGridLayout()

        HBox1 = QHBoxLayout()
        HBox2 = QHBoxLayout()

        label_title = QLabel()
        label_title.setText('<center><font size="7">Choose result file</font></center>')

        Next_button = QPushButton("&Next")
        Next_button.clicked.connect(self.pass_result)
        Back_button = QPushButton("&Back")
        Back_button.clicked.connect(self.back_page)

        Choose_button = QPushButton("Choose file")
        Choose_button.clicked.connect(self.choose_file)

        bottom_box.addWidget(Back_button)
        bottom_box.addStretch()
        bottom_box.addWidget(Next_button)

        label_result = QLabel()
        label_result.setText("I-V scan Result file : ")
        label_comment = QLabel()
        label_comment.setText("Comment : ")

        self.edit_comment = QTextEdit()
        self.edit_comment.setText(self.parent.result_info["comment"])
        self.edit_result = QLineEdit()
        self.edit_result.setFocusPolicy(Qt.NoFocus)
        self.edit_result.setReadOnly(True)
        self.edit_result.setFixedWidth(500)
        if self.parent.result_info["filename"] != "":
            self.edit_result.setText(self.parent.result_info["filename"])
        else:
            self.edit_result.setText("")

        HBox1.addWidget(self.edit_result)
        HBox1.addWidget(Choose_button)

        grid_edit.addWidget(label_result, 0, 0)
        grid_edit.addLayout(HBox1, 0, 1)

        grid_edit.addWidget(label_comment, 1, 0)

        outer = QScrollArea()
        outer.setWidgetResizable(True)
        outer.setWidget(self.edit_comment)
        HBox2.addWidget(outer)
        grid_edit.addWidget(outer, 1, 1)

        layout.addWidget(label_title)
        layout.addStretch()
        layout.addLayout(grid_edit)
        layout.addLayout(bottom_box)

        self.setLayout(layout)

        self.parent.parent.atlsn_sen = function.get_sensor(
            self.parent.parent, self.parent.parent.info_dict["component"]
        )
        self.parent.parent.dict_sen = function.fill_info_sensor(
            self.parent.parent,
            self.parent.parent.atlsn_sen,
            self.parent.parent.info_dict,
        )
        self.parent.parent.testRun_sen = function.get_testRun(
            self.parent.parent,
            self.parent.parent.dict_sen["component"],
            self.parent.parent.version,
            self.parent.parent.atlsn_sen,
        )

    def choose_file(self):
        dlg = QFileDialog()
        dlg.setFileMode(QFileDialog.AnyFile)

        try:
            dlg.exec_()
            filename = dlg.selectedFiles()[0]
            self.edit_result.setText(filename)
        except Exception:
            log.exception(traceback.format_exc())
            QMessageBox.warning(
                None, "Warning", "Please choose correct file", QMessageBox.Ok
            )

    def pass_result(self):
        try:
            self.parent.result_info["filename"] = self.edit_result.text()
            if ".dat" in self.edit_result.text():
                get_data = function.read_data(self.edit_result.text())

                result_dict = {
                    "IV_ARRAY": {
                        "time": get_data.get("time"),
                        "voltage": get_data.get("voltage"),
                        "current": get_data.get("current"),
                        "sigma current": get_data.get("sigma"),
                        "temperature": get_data.get("temp"),
                        "humidity": get_data.get("hum"),
                    }
                }
            else:
                with Path(self.edit_result.text()).open(encoding="utf-8") as f:
                    result_dict = json.load(f)

                with Path(self.jsonschema_path).open(encoding="utf-8") as f:
                    jsonschema_dict = json.load(f)
                jsonschema.validate(result_dict, jsonschema_dict)

            if result_dict["IV_ARRAY"].get("humidity") is None:
                result_dict["IV_ARRAY"]["humidity"] = [0] * len(
                    result_dict["IV_ARRAY"]["voltage"]
                )
            if result_dict["IV_ARRAY"]["sigma current"] is None:
                result_dict["IV_ARRAY"]["sigma current"] = [0] * len(
                    result_dict["IV_ARRAY"]["voltage"]
                )
            free_comment = self.edit_comment.toPlainText()

            self.parent.parent.testRun_sen["results"]["IV_ARRAY"] = result_dict[
                "IV_ARRAY"
            ]

            nElems = [
                len(result_dict["IV_ARRAY"][v])
                for v in [
                    "voltage",
                    "current",
                    "temperature",
                    "sigma current",
                    "humidity",
                    "time",
                ]
            ]

            if sum(int(nElems[0] == v) for v in nElems) < len(nElems):
                msg = "Format error: array sizes do not match each other in IV_ARRAY"
                raise Exception(msg)

            if "properties" in result_dict:
                for var in ["TEMP", "HUM"]:
                    self.parent.parent.testRun_sen["results"]["property"][
                        var
                    ] = result_dict["properties"].get(var)

            self.parent.parent.testRun_sen["results"]["comment"] = str(free_comment)

            self.hide()

            # self.parent.parent.receive_result_IV(self)
            function.receive_result_IV(self.parent.parent, self)

        except jsonschema.exceptions.ValidationError:
            log.exception(traceback.format_exc())
            QMessageBox.warning(
                None, "Warning", "File format not correct", QMessageBox.Ok
            )
        except FileNotFoundError:
            log.exception(traceback.format_exc())
            QMessageBox.warning(None, "Warning", "File not found", QMessageBox.Ok)
        except Exception:
            log.exception(traceback.format_exc())
            QMessageBox.warning(None, "Warning", "Unexpected Error", QMessageBox.Ok)

    def back_page(self):
        self.parent.close_and_return()



from __future__ import annotations

from PyQt5.QtWidgets import (
    QCheckBox,
    QGridLayout,
    QLabel,
    QLineEdit,
    QPushButton,
    QVBoxLayout,
)


def commonpart_gen(w):
    layout = QGridLayout()
    label_comment = QLabel()
    label_comment.setText("Comment:")
    layout.addWidget(label_comment, 0, 0, 1, 2)
    w.edit_comment = QLineEdit()
    layout.addWidget(w.edit_comment, 1, 0, 2, 2)
    button_checked = QPushButton("Checked")
    button_checked.clicked.connect(w.checked)
    layout.addWidget(button_checked, 3, 1)

    button_back = QPushButton("Back")
    button_back.clicked.connect(w.back_page)
    button_next = QPushButton("Next")
    button_next.clicked.connect(w.next_page)
    layout.addWidget(button_back, 4, 0)
    layout.addWidget(button_next, 4, 1)

    return layout


def checklist_gen(w, page, tot_page):
    layout = QVBoxLayout()
    layout_child = QGridLayout()
    label_page = QLabel()
    label_page.setText(f"page: {page + 1}/{tot_page}")
    layout.addWidget(label_page)
    label = QLabel()
    label.setText("If there are any problems, check in checkbox.")
    layout.addWidget(label)

    if page == 0:
        w.cb_Wire_0_0 = QCheckBox("Wire_0_0", w)
        w.cb_Wire_0_1 = QCheckBox("Wire_0_1", w)
        w.cb_Wire_0_2 = QCheckBox("Wire_0_2", w)
        w.cb_Wire_0_3 = QCheckBox("Wire_0_3", w)
        w.cb_Wire_0_4 = QCheckBox("Wire_0_4", w)
        w.cb_Wire_0_5 = QCheckBox("Wire_0_5", w)
        w.cb_Wire_0_6 = QCheckBox("Wire_0_6", w)
        w.cb_Wire_0_7 = QCheckBox("Wire_0_7", w)
        w.cb_Wire_0_8 = QCheckBox("Wire_0_8", w)
        w.cb_Wire_0_9 = QCheckBox("Wire_0_9", w)
        w.cb_Wire_0_10 = QCheckBox("Wire_0_10", w)

        w.cb_Wire_0_0.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_0_0.checkState(), "Wire_0_0")
        )
        w.cb_Wire_0_1.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_0_1.checkState(), "Wire_0_1")
        )
        w.cb_Wire_0_2.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_0_2.checkState(), "Wire_0_2")
        )
        w.cb_Wire_0_3.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_0_3.checkState(), "Wire_0_3")
        )
        w.cb_Wire_0_4.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_0_4.checkState(), "Wire_0_4")
        )
        w.cb_Wire_0_5.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_0_5.checkState(), "Wire_0_5")
        )
        w.cb_Wire_0_6.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_0_6.checkState(), "Wire_0_6")
        )
        w.cb_Wire_0_7.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_0_7.checkState(), "Wire_0_7")
        )
        w.cb_Wire_0_8.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_0_8.checkState(), "Wire_0_8")
        )
        w.cb_Wire_0_9.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_0_9.checkState(), "Wire_0_9")
        )
        w.cb_Wire_0_10.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_0_10.checkState(), "Wire_0_10")
        )

        layout_child.addWidget(w.cb_Wire_0_0, 0, 0)
        layout_child.addWidget(w.cb_Wire_0_1, 1, 0)
        layout_child.addWidget(w.cb_Wire_0_2, 2, 0)
        layout_child.addWidget(w.cb_Wire_0_3, 3, 0)
        layout_child.addWidget(w.cb_Wire_0_4, 4, 0)
        layout_child.addWidget(w.cb_Wire_0_5, 5, 0)
        layout_child.addWidget(w.cb_Wire_0_6, 6, 0)
        layout_child.addWidget(w.cb_Wire_0_7, 7, 0)
        layout_child.addWidget(w.cb_Wire_0_8, 8, 0)
        layout_child.addWidget(w.cb_Wire_0_9, 9, 0)
        layout_child.addWidget(w.cb_Wire_0_10, 10, 0)

        layout.addLayout(layout_child)
        layout.addLayout(commonpart_gen(w))

    if page == 1:
        layout.addLayout(layout_child)
        layout.addLayout(commonpart_gen(w))

    if page == 2:
        w.cb_Wire_2_0 = QCheckBox("Wire_2_0", w)

        w.cb_Wire_2_0.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_2_0.checkState(), "Wire_2_0")
        )

        layout_child.addWidget(w.cb_Wire_2_0)

        layout.addLayout(layout_child)
        layout.addLayout(commonpart_gen(w))

    if page == 3:
        layout.addLayout(layout_child)
        layout.addLayout(commonpart_gen(w))

    if page == 4:
        layout.addLayout(layout_child)
        layout.addLayout(commonpart_gen(w))

    if page == 5:
        w.cb_Wire_5_0 = QCheckBox("Wire_5_0", w)
        w.cb_Wire_5_1 = QCheckBox("Wire_5_1", w)
        w.cb_Wire_5_2 = QCheckBox("Wire_5_2", w)
        w.cb_Wire_5_3 = QCheckBox("Wire_5_3", w)
        w.cb_Wire_5_4 = QCheckBox("Wire_5_4", w)
        w.cb_Wire_5_5 = QCheckBox("Wire_5_5", w)
        w.cb_Wire_5_6 = QCheckBox("Wire_5_6", w)
        w.cb_Wire_5_7 = QCheckBox("Wire_5_7", w)
        w.cb_Wire_5_8 = QCheckBox("Wire_5_8", w)

        w.cb_Wire_5_0.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_5_0.checkState(), "Wire_5_0")
        )
        w.cb_Wire_5_1.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_5_1.checkState(), "Wire_5_1")
        )
        w.cb_Wire_5_2.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_5_2.checkState(), "Wire_5_2")
        )
        w.cb_Wire_5_3.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_5_3.checkState(), "Wire_5_3")
        )
        w.cb_Wire_5_4.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_5_4.checkState(), "Wire_5_4")
        )
        w.cb_Wire_5_5.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_5_5.checkState(), "Wire_5_5")
        )
        w.cb_Wire_5_6.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_5_6.checkState(), "Wire_5_6")
        )
        w.cb_Wire_5_7.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_5_7.checkState(), "Wire_5_7")
        )
        w.cb_Wire_5_8.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_5_8.checkState(), "Wire_5_8")
        )

        layout_child.addWidget(w.cb_Wire_5_0, 0, 0)
        layout_child.addWidget(w.cb_Wire_5_1, 1, 0)
        layout_child.addWidget(w.cb_Wire_5_2, 2, 0)
        layout_child.addWidget(w.cb_Wire_5_3, 3, 0)
        layout_child.addWidget(w.cb_Wire_5_4, 4, 0)
        layout_child.addWidget(w.cb_Wire_5_5, 5, 0)
        layout_child.addWidget(w.cb_Wire_5_6, 6, 0)
        layout_child.addWidget(w.cb_Wire_5_7, 7, 0)
        layout_child.addWidget(w.cb_Wire_5_8, 8, 0)

        layout.addLayout(layout_child)
        layout.addLayout(commonpart_gen(w))

    if page == 6:
        w.cb_Wire_6_0 = QCheckBox("Wire_6_0", w)
        w.cb_Wire_6_1 = QCheckBox("Wire_6_1", w)
        w.cb_Wire_6_2 = QCheckBox("Wire_6_2", w)
        w.cb_Wire_6_3 = QCheckBox("Wire_6_3", w)
        w.cb_Wire_6_4 = QCheckBox("Wire_6_4", w)
        w.cb_Wire_6_5 = QCheckBox("Wire_6_5", w)
        w.cb_Wire_6_6 = QCheckBox("Wire_6_6", w)
        w.cb_Wire_6_7 = QCheckBox("Wire_6_7", w)
        w.cb_Wire_6_8 = QCheckBox("Wire_6_8", w)
        w.cb_Wire_6_9 = QCheckBox("Wire_6_9", w)
        w.cb_Wire_6_10 = QCheckBox("Wire_6_10", w)

        w.cb_Wire_6_0.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_6_0.checkState(), "Wire_6_0")
        )
        w.cb_Wire_6_1.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_6_1.checkState(), "Wire_6_1")
        )
        w.cb_Wire_6_2.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_6_2.checkState(), "Wire_6_2")
        )
        w.cb_Wire_6_3.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_6_3.checkState(), "Wire_6_3")
        )
        w.cb_Wire_6_4.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_6_4.checkState(), "Wire_6_4")
        )
        w.cb_Wire_6_5.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_6_5.checkState(), "Wire_6_5")
        )
        w.cb_Wire_6_6.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_6_6.checkState(), "Wire_6_6")
        )
        w.cb_Wire_6_7.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_6_7.checkState(), "Wire_6_7")
        )
        w.cb_Wire_6_8.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_6_8.checkState(), "Wire_6_8")
        )
        w.cb_Wire_6_9.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_6_9.checkState(), "Wire_6_9")
        )
        w.cb_Wire_6_10.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_6_10.checkState(), "Wire_6_10")
        )

        layout_child.addWidget(w.cb_Wire_6_0, 0, 0)
        layout_child.addWidget(w.cb_Wire_6_1, 1, 0)
        layout_child.addWidget(w.cb_Wire_6_2, 2, 0)
        layout_child.addWidget(w.cb_Wire_6_3, 3, 0)
        layout_child.addWidget(w.cb_Wire_6_4, 4, 0)
        layout_child.addWidget(w.cb_Wire_6_5, 5, 0)
        layout_child.addWidget(w.cb_Wire_6_6, 6, 0)
        layout_child.addWidget(w.cb_Wire_6_7, 7, 0)
        layout_child.addWidget(w.cb_Wire_6_8, 8, 0)
        layout_child.addWidget(w.cb_Wire_6_9, 9, 0)
        layout_child.addWidget(w.cb_Wire_6_10, 10, 0)

        layout.addLayout(layout_child)
        layout.addLayout(commonpart_gen(w))

    if page == 7:
        layout.addLayout(layout_child)
        layout.addLayout(commonpart_gen(w))

    if page == 8:
        layout.addLayout(layout_child)
        layout.addLayout(commonpart_gen(w))

    if page == 9:
        layout.addLayout(layout_child)
        layout.addLayout(commonpart_gen(w))

    if page == 10:
        layout.addLayout(layout_child)
        layout.addLayout(commonpart_gen(w))

    if page == 11:
        w.cb_Wire_11_0 = QCheckBox("Wire_11_0", w)
        w.cb_Wire_11_1 = QCheckBox("Wire_11_1", w)
        w.cb_Wire_11_2 = QCheckBox("Wire_11_2", w)
        w.cb_Wire_11_3 = QCheckBox("Wire_11_3", w)
        w.cb_Wire_11_4 = QCheckBox("Wire_11_4", w)
        w.cb_Wire_11_5 = QCheckBox("Wire_11_5", w)
        w.cb_Wire_11_6 = QCheckBox("Wire_11_6", w)
        w.cb_Wire_11_7 = QCheckBox("Wire_11_7", w)
        w.cb_Wire_11_8 = QCheckBox("Wire_11_8", w)
        w.cb_Wire_11_9 = QCheckBox("Wire_11_9", w)

        w.cb_Wire_11_0.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_11_0.checkState(), "Wire_11_0")
        )
        w.cb_Wire_11_1.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_11_1.checkState(), "Wire_11_1")
        )
        w.cb_Wire_11_2.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_11_2.checkState(), "Wire_11_2")
        )
        w.cb_Wire_11_3.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_11_3.checkState(), "Wire_11_3")
        )
        w.cb_Wire_11_4.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_11_4.checkState(), "Wire_11_4")
        )
        w.cb_Wire_11_5.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_11_5.checkState(), "Wire_11_5")
        )
        w.cb_Wire_11_6.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_11_6.checkState(), "Wire_11_6")
        )
        w.cb_Wire_11_7.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_11_7.checkState(), "Wire_11_7")
        )
        w.cb_Wire_11_8.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_11_8.checkState(), "Wire_11_8")
        )
        w.cb_Wire_11_9.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_11_9.checkState(), "Wire_11_9")
        )

        layout_child.addWidget(w.cb_Wire_11_0, 0, 0)
        layout_child.addWidget(w.cb_Wire_11_1, 1, 0)
        layout_child.addWidget(w.cb_Wire_11_2, 2, 0)
        layout_child.addWidget(w.cb_Wire_11_3, 3, 0)
        layout_child.addWidget(w.cb_Wire_11_4, 4, 0)
        layout_child.addWidget(w.cb_Wire_11_5, 5, 0)
        layout_child.addWidget(w.cb_Wire_11_6, 6, 0)
        layout_child.addWidget(w.cb_Wire_11_7, 7, 0)
        layout_child.addWidget(w.cb_Wire_11_8, 8, 0)
        layout_child.addWidget(w.cb_Wire_11_9, 9, 0)

        layout.addLayout(layout_child)
        layout.addLayout(commonpart_gen(w))

    if page == 12:
        w.cb_Wire_12_0 = QCheckBox("Wire_12_0", w)
        w.cb_Wire_12_1 = QCheckBox("Wire_12_1", w)
        w.cb_Wire_12_2 = QCheckBox("Wire_12_2", w)
        w.cb_Wire_12_3 = QCheckBox("Wire_12_3", w)
        w.cb_Wire_12_4 = QCheckBox("Wire_12_4", w)
        w.cb_Wire_12_5 = QCheckBox("Wire_12_5", w)
        w.cb_Wire_12_6 = QCheckBox("Wire_12_6", w)
        w.cb_Wire_12_7 = QCheckBox("Wire_12_7", w)
        w.cb_Wire_12_8 = QCheckBox("Wire_12_8", w)

        w.cb_Wire_12_0.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_12_0.checkState(), "Wire_12_0")
        )
        w.cb_Wire_12_1.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_12_1.checkState(), "Wire_12_1")
        )
        w.cb_Wire_12_2.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_12_2.checkState(), "Wire_12_2")
        )
        w.cb_Wire_12_3.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_12_3.checkState(), "Wire_12_3")
        )
        w.cb_Wire_12_4.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_12_4.checkState(), "Wire_12_4")
        )
        w.cb_Wire_12_5.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_12_5.checkState(), "Wire_12_5")
        )
        w.cb_Wire_12_6.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_12_6.checkState(), "Wire_12_6")
        )
        w.cb_Wire_12_7.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_12_7.checkState(), "Wire_12_7")
        )
        w.cb_Wire_12_8.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_12_8.checkState(), "Wire_12_8")
        )

        layout_child.addWidget(w.cb_Wire_12_0, 0, 0)
        layout_child.addWidget(w.cb_Wire_12_1, 1, 0)
        layout_child.addWidget(w.cb_Wire_12_2, 2, 0)
        layout_child.addWidget(w.cb_Wire_12_3, 3, 0)
        layout_child.addWidget(w.cb_Wire_12_4, 4, 0)
        layout_child.addWidget(w.cb_Wire_12_5, 5, 0)
        layout_child.addWidget(w.cb_Wire_12_6, 6, 0)
        layout_child.addWidget(w.cb_Wire_12_7, 7, 0)
        layout_child.addWidget(w.cb_Wire_12_8, 8, 0)

        layout.addLayout(layout_child)
        layout.addLayout(commonpart_gen(w))

    if page == 13:
        layout.addLayout(layout_child)
        layout.addLayout(commonpart_gen(w))

    if page == 14:
        layout.addLayout(layout_child)
        layout.addLayout(commonpart_gen(w))

    if page == 15:
        layout.addLayout(layout_child)
        layout.addLayout(commonpart_gen(w))

    if page == 16:
        layout.addLayout(layout_child)
        layout.addLayout(commonpart_gen(w))

    if page == 17:
        w.cb_Wire_17_0 = QCheckBox("Wire_17_0", w)
        w.cb_Wire_17_1 = QCheckBox("Wire_17_1", w)
        w.cb_Wire_17_2 = QCheckBox("Wire_17_2", w)
        w.cb_Wire_17_3 = QCheckBox("Wire_17_3", w)
        w.cb_Wire_17_4 = QCheckBox("Wire_17_4", w)
        w.cb_Wire_17_5 = QCheckBox("Wire_17_5", w)
        w.cb_Wire_17_6 = QCheckBox("Wire_17_6", w)
        w.cb_Wire_17_7 = QCheckBox("Wire_17_7", w)
        w.cb_Wire_17_8 = QCheckBox("Wire_17_8", w)
        w.cb_Wire_17_9 = QCheckBox("Wire_17_9", w)
        w.cb_Wire_17_10 = QCheckBox("Wire_17_10", w)
        w.cb_Wire_17_11 = QCheckBox("Wire_17_11", w)

        w.cb_Wire_17_0.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_17_0.checkState(), "Wire_17_0")
        )
        w.cb_Wire_17_1.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_17_1.checkState(), "Wire_17_1")
        )
        w.cb_Wire_17_2.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_17_2.checkState(), "Wire_17_2")
        )
        w.cb_Wire_17_3.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_17_3.checkState(), "Wire_17_3")
        )
        w.cb_Wire_17_4.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_17_4.checkState(), "Wire_17_4")
        )
        w.cb_Wire_17_5.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_17_5.checkState(), "Wire_17_5")
        )
        w.cb_Wire_17_6.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_17_6.checkState(), "Wire_17_6")
        )
        w.cb_Wire_17_7.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_17_7.checkState(), "Wire_17_7")
        )
        w.cb_Wire_17_8.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_17_8.checkState(), "Wire_17_8")
        )
        w.cb_Wire_17_9.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_17_9.checkState(), "Wire_17_9")
        )
        w.cb_Wire_17_10.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_17_10.checkState(), "Wire_17_10")
        )
        w.cb_Wire_17_11.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_17_11.checkState(), "Wire_17_11")
        )

        layout_child.addWidget(w.cb_Wire_17_0, 0, 0)
        layout_child.addWidget(w.cb_Wire_17_1, 1, 0)
        layout_child.addWidget(w.cb_Wire_17_2, 2, 0)
        layout_child.addWidget(w.cb_Wire_17_3, 3, 0)
        layout_child.addWidget(w.cb_Wire_17_4, 4, 0)
        layout_child.addWidget(w.cb_Wire_17_5, 5, 0)
        layout_child.addWidget(w.cb_Wire_17_6, 6, 0)
        layout_child.addWidget(w.cb_Wire_17_7, 7, 0)
        layout_child.addWidget(w.cb_Wire_17_8, 8, 0)
        layout_child.addWidget(w.cb_Wire_17_9, 9, 0)
        layout_child.addWidget(w.cb_Wire_17_10, 10, 0)
        layout_child.addWidget(w.cb_Wire_17_11, 11, 0)

        layout.addLayout(layout_child)
        layout.addLayout(commonpart_gen(w))

    if page == 18:
        w.cb_Wire_18_0 = QCheckBox("Wire_18_0", w)
        w.cb_Wire_18_1 = QCheckBox("Wire_18_1", w)
        w.cb_Wire_18_2 = QCheckBox("Wire_18_2", w)
        w.cb_Wire_18_3 = QCheckBox("Wire_18_3", w)
        w.cb_Wire_18_4 = QCheckBox("Wire_18_4", w)
        w.cb_Wire_18_5 = QCheckBox("Wire_18_5", w)
        w.cb_Wire_18_6 = QCheckBox("Wire_18_6", w)
        w.cb_Wire_18_7 = QCheckBox("Wire_18_7", w)
        w.cb_Wire_18_8 = QCheckBox("Wire_18_8", w)
        w.cb_Wire_18_9 = QCheckBox("Wire_18_9", w)
        w.cb_Wire_18_10 = QCheckBox("Wire_18_10", w)
        w.cb_Wire_18_11 = QCheckBox("Wire_18_11", w)

        w.cb_Wire_18_0.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_18_0.checkState(), "Wire_18_0")
        )
        w.cb_Wire_18_1.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_18_1.checkState(), "Wire_18_1")
        )
        w.cb_Wire_18_2.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_18_2.checkState(), "Wire_18_2")
        )
        w.cb_Wire_18_3.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_18_3.checkState(), "Wire_18_3")
        )
        w.cb_Wire_18_4.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_18_4.checkState(), "Wire_18_4")
        )
        w.cb_Wire_18_5.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_18_5.checkState(), "Wire_18_5")
        )
        w.cb_Wire_18_6.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_18_6.checkState(), "Wire_18_6")
        )
        w.cb_Wire_18_7.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_18_7.checkState(), "Wire_18_7")
        )
        w.cb_Wire_18_8.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_18_8.checkState(), "Wire_18_8")
        )
        w.cb_Wire_18_9.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_18_9.checkState(), "Wire_18_9")
        )
        w.cb_Wire_18_10.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_18_10.checkState(), "Wire_18_10")
        )
        w.cb_Wire_18_11.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_18_11.checkState(), "Wire_18_11")
        )

        layout_child.addWidget(w.cb_Wire_18_0, 0, 0)
        layout_child.addWidget(w.cb_Wire_18_1, 1, 0)
        layout_child.addWidget(w.cb_Wire_18_2, 2, 0)
        layout_child.addWidget(w.cb_Wire_18_3, 3, 0)
        layout_child.addWidget(w.cb_Wire_18_4, 4, 0)
        layout_child.addWidget(w.cb_Wire_18_5, 5, 0)
        layout_child.addWidget(w.cb_Wire_18_6, 6, 0)
        layout_child.addWidget(w.cb_Wire_18_7, 7, 0)
        layout_child.addWidget(w.cb_Wire_18_8, 8, 0)
        layout_child.addWidget(w.cb_Wire_18_9, 9, 0)
        layout_child.addWidget(w.cb_Wire_18_10, 10, 0)
        layout_child.addWidget(w.cb_Wire_18_11, 11, 0)

        layout.addLayout(layout_child)
        layout.addLayout(commonpart_gen(w))

    if page == 19:
        layout.addLayout(layout_child)
        layout.addLayout(commonpart_gen(w))

    if page == 20:
        layout.addLayout(layout_child)
        layout.addLayout(commonpart_gen(w))

    if page == 21:
        layout.addLayout(layout_child)
        layout.addLayout(commonpart_gen(w))

    if page == 22:
        layout.addLayout(layout_child)
        layout.addLayout(commonpart_gen(w))

    if page == 23:
        w.cb_Wire_23_0 = QCheckBox("Wire_23_0", w)
        w.cb_Wire_23_1 = QCheckBox("Wire_23_1", w)
        w.cb_Wire_23_2 = QCheckBox("Wire_23_2", w)
        w.cb_Wire_23_3 = QCheckBox("Wire_23_3", w)
        w.cb_Wire_23_4 = QCheckBox("Wire_23_4", w)
        w.cb_Wire_23_5 = QCheckBox("Wire_23_5", w)
        w.cb_Wire_23_6 = QCheckBox("Wire_23_6", w)
        w.cb_Wire_23_7 = QCheckBox("Wire_23_7", w)
        w.cb_Wire_23_8 = QCheckBox("Wire_23_8", w)

        w.cb_Wire_23_0.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_23_0.checkState(), "Wire_23_0")
        )
        w.cb_Wire_23_1.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_23_0.checkState(), "Wire_23_1")
        )
        w.cb_Wire_23_2.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_23_1.checkState(), "Wire_23_2")
        )
        w.cb_Wire_23_3.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_23_2.checkState(), "Wire_23_3")
        )
        w.cb_Wire_23_4.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_23_3.checkState(), "Wire_23_4")
        )
        w.cb_Wire_23_5.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_23_4.checkState(), "Wire_23_5")
        )
        w.cb_Wire_23_6.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_23_5.checkState(), "Wire_23_6")
        )
        w.cb_Wire_23_7.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_23_6.checkState(), "Wire_23_7")
        )
        w.cb_Wire_23_8.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_23_7.checkState(), "Wire_23_8")
        )

        layout_child.addWidget(w.cb_Wire_23_0, 0, 0)
        layout_child.addWidget(w.cb_Wire_23_1, 1, 0)
        layout_child.addWidget(w.cb_Wire_23_2, 2, 0)
        layout_child.addWidget(w.cb_Wire_23_3, 3, 0)
        layout_child.addWidget(w.cb_Wire_23_4, 4, 0)
        layout_child.addWidget(w.cb_Wire_23_5, 5, 0)
        layout_child.addWidget(w.cb_Wire_23_6, 6, 0)
        layout_child.addWidget(w.cb_Wire_23_7, 7, 0)
        layout_child.addWidget(w.cb_Wire_23_8, 8, 0)

        layout.addLayout(layout_child)
        layout.addLayout(commonpart_gen(w))

    if page == 24:
        w.cb_Wire_24_0 = QCheckBox("Wire_24_0", w)
        w.cb_Wire_24_1 = QCheckBox("Wire_24_1", w)
        w.cb_Wire_24_2 = QCheckBox("Wire_24_2", w)
        w.cb_Wire_24_3 = QCheckBox("Wire_24_3", w)
        w.cb_Wire_24_4 = QCheckBox("Wire_24_4", w)
        w.cb_Wire_24_5 = QCheckBox("Wire_24_5", w)
        w.cb_Wire_24_6 = QCheckBox("Wire_24_6", w)
        w.cb_Wire_24_7 = QCheckBox("Wire_24_7", w)
        w.cb_Wire_24_8 = QCheckBox("Wire_24_8", w)
        w.cb_Wire_24_9 = QCheckBox("Wire_24_9", w)
        w.cb_Wire_24_10 = QCheckBox("Wire_24_10", w)
        w.cb_Wire_24_11 = QCheckBox("Wire_24_11", w)

        w.cb_Wire_24_0.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_24_0.checkState(), "Wire_24_0")
        )
        w.cb_Wire_24_1.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_24_1.checkState(), "Wire_24_1")
        )
        w.cb_Wire_24_2.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_24_2.checkState(), "Wire_24_2")
        )
        w.cb_Wire_24_3.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_24_3.checkState(), "Wire_24_3")
        )
        w.cb_Wire_24_4.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_24_4.checkState(), "Wire_24_4")
        )
        w.cb_Wire_24_5.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_24_5.checkState(), "Wire_24_5")
        )
        w.cb_Wire_24_6.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_24_6.checkState(), "Wire_24_6")
        )
        w.cb_Wire_24_7.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_24_7.checkState(), "Wire_24_7")
        )
        w.cb_Wire_24_8.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_24_8.checkState(), "Wire_24_8")
        )
        w.cb_Wire_24_9.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_24_9.checkState(), "Wire_24_9")
        )
        w.cb_Wire_24_10.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_24_10.checkState(), "Wire_24_10")
        )
        w.cb_Wire_24_11.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_24_11.checkState(), "Wire_24_11")
        )

        layout_child.addWidget(w.cb_Wire_24_0, 0, 0)
        layout_child.addWidget(w.cb_Wire_24_1, 1, 0)
        layout_child.addWidget(w.cb_Wire_24_2, 2, 0)
        layout_child.addWidget(w.cb_Wire_24_3, 3, 0)
        layout_child.addWidget(w.cb_Wire_24_4, 4, 0)
        layout_child.addWidget(w.cb_Wire_24_5, 5, 0)
        layout_child.addWidget(w.cb_Wire_24_6, 6, 0)
        layout_child.addWidget(w.cb_Wire_24_7, 7, 0)
        layout_child.addWidget(w.cb_Wire_24_8, 8, 0)
        layout_child.addWidget(w.cb_Wire_24_9, 9, 0)
        layout_child.addWidget(w.cb_Wire_24_10, 10, 0)
        layout_child.addWidget(w.cb_Wire_24_11, 11, 0)

        layout.addLayout(layout_child)
        layout.addLayout(commonpart_gen(w))

    if page == 25:
        layout.addLayout(layout_child)
        layout.addLayout(commonpart_gen(w))

    if page == 26:
        layout.addLayout(layout_child)
        layout.addLayout(commonpart_gen(w))

    if page == 27:
        layout.addLayout(layout_child)
        layout.addLayout(commonpart_gen(w))

    if page == 28:
        layout.addLayout(layout_child)
        layout.addLayout(commonpart_gen(w))

    if page == 29:
        w.cb_Wire_29_0 = QCheckBox("Wire_29_0", w)
        w.cb_Wire_29_1 = QCheckBox("Wire_29_1", w)
        w.cb_Wire_29_2 = QCheckBox("Wire_29_2", w)
        w.cb_Wire_29_3 = QCheckBox("Wire_29_3", w)
        w.cb_Wire_29_4 = QCheckBox("Wire_29_4", w)
        w.cb_Wire_29_5 = QCheckBox("Wire_29_5", w)
        w.cb_Wire_29_6 = QCheckBox("Wire_29_6", w)
        w.cb_Wire_29_7 = QCheckBox("Wire_29_7", w)
        w.cb_Wire_29_8 = QCheckBox("Wire_29_8", w)
        w.cb_Wire_29_9 = QCheckBox("Wire_29_9", w)
        w.cb_Wire_29_10 = QCheckBox("Wire_29_10", w)
        w.cb_Wire_29_11 = QCheckBox("Wire_29_11", w)

        w.cb_Wire_29_0.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_29_0.checkState(), "Wire_29_0")
        )
        w.cb_Wire_29_1.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_29_1.checkState(), "Wire_29_1")
        )
        w.cb_Wire_29_2.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_29_2.checkState(), "Wire_29_2")
        )
        w.cb_Wire_29_3.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_29_3.checkState(), "Wire_29_3")
        )
        w.cb_Wire_29_4.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_29_4.checkState(), "Wire_29_4")
        )
        w.cb_Wire_29_5.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_29_5.checkState(), "Wire_29_5")
        )
        w.cb_Wire_29_6.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_29_6.checkState(), "Wire_29_6")
        )
        w.cb_Wire_29_7.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_29_7.checkState(), "Wire_29_7")
        )
        w.cb_Wire_29_8.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_29_8.checkState(), "Wire_29_8")
        )
        w.cb_Wire_29_9.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_29_9.checkState(), "Wire_29_9")
        )
        w.cb_Wire_29_10.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_29_10.checkState(), "Wire_29_10")
        )
        w.cb_Wire_29_11.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_29_11.checkState(), "Wire_29_11")
        )

        layout_child.addWidget(w.cb_Wire_29_0, 0, 0)
        layout_child.addWidget(w.cb_Wire_29_1, 1, 0)
        layout_child.addWidget(w.cb_Wire_29_2, 2, 0)
        layout_child.addWidget(w.cb_Wire_29_3, 3, 0)
        layout_child.addWidget(w.cb_Wire_29_4, 4, 0)
        layout_child.addWidget(w.cb_Wire_29_5, 5, 0)
        layout_child.addWidget(w.cb_Wire_29_6, 6, 0)
        layout_child.addWidget(w.cb_Wire_29_7, 7, 0)
        layout_child.addWidget(w.cb_Wire_29_8, 8, 0)
        layout_child.addWidget(w.cb_Wire_29_9, 9, 0)
        layout_child.addWidget(w.cb_Wire_29_10, 10, 0)
        layout_child.addWidget(w.cb_Wire_29_11, 11, 0)

        layout.addLayout(layout_child)
        layout.addLayout(commonpart_gen(w))

    if page == 30:
        w.cb_Wire_30_0 = QCheckBox("Wire_30_0", w)
        w.cb_Wire_30_1 = QCheckBox("Wire_30_1", w)
        w.cb_Wire_30_2 = QCheckBox("Wire_30_2", w)
        w.cb_Wire_30_3 = QCheckBox("Wire_30_3", w)
        w.cb_Wire_30_4 = QCheckBox("Wire_30_4", w)
        w.cb_Wire_30_5 = QCheckBox("Wire_30_5", w)
        w.cb_Wire_30_6 = QCheckBox("Wire_30_6", w)

        w.cb_Wire_30_0.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_30_0.checkState(), "Wire_30_0")
        )
        w.cb_Wire_30_1.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_30_1.checkState(), "Wire_30_1")
        )
        w.cb_Wire_30_2.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_30_2.checkState(), "Wire_30_2")
        )
        w.cb_Wire_30_3.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_30_3.checkState(), "Wire_30_3")
        )
        w.cb_Wire_30_4.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_30_4.checkState(), "Wire_30_4")
        )
        w.cb_Wire_30_5.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_30_5.checkState(), "Wire_30_5")
        )
        w.cb_Wire_30_6.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_30_6.checkState(), "Wire_30_6")
        )

        layout_child.addWidget(w.cb_Wire_30_0, 0, 0)
        layout_child.addWidget(w.cb_Wire_30_1, 1, 0)
        layout_child.addWidget(w.cb_Wire_30_2, 2, 0)
        layout_child.addWidget(w.cb_Wire_30_3, 3, 0)
        layout_child.addWidget(w.cb_Wire_30_4, 4, 0)
        layout_child.addWidget(w.cb_Wire_30_5, 5, 0)
        layout_child.addWidget(w.cb_Wire_30_6, 6, 0)

        layout.addLayout(layout_child)
        layout.addLayout(commonpart_gen(w))

    if page == 31:
        w.cb_Wire_31_0 = QCheckBox("Wire_31_0", w)

        w.cb_SMD_31_0.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_SMD_31_0.checkState(), "SMD_31_0")
        )

        layout_child.addWidget(w.cb_Wire_31_0, 0, 0)

        layout.addLayout(layout_child)
        layout.addLayout(commonpart_gen(w))

    if page == 32:
        layout.addLayout(layout_child)
        layout.addLayout(commonpart_gen(w))

    if page == 33:
        layout.addLayout(layout_child)
        layout.addLayout(commonpart_gen(w))

    if page == 34:
        layout.addLayout(layout_child)
        layout.addLayout(commonpart_gen(w))

    if page == 35:
        w.cb_Wire_35_0 = QCheckBox("Wire_35_0", w)
        w.cb_Wire_35_1 = QCheckBox("Wire_35_1", w)
        w.cb_Wire_35_2 = QCheckBox("Wire_35_2", w)
        w.cb_Wire_35_3 = QCheckBox("Wire_35_3", w)
        w.cb_Wire_35_4 = QCheckBox("Wire_35_4", w)
        w.cb_Wire_35_5 = QCheckBox("Wire_35_5", w)
        w.cb_Wire_35_6 = QCheckBox("Wire_35_6", w)
        w.cb_Wire_35_7 = QCheckBox("Wire_35_7", w)
        w.cb_Wire_35_8 = QCheckBox("Wire_35_8", w)
        w.cb_Wire_35_9 = QCheckBox("Wire_35_9", w)

        w.cb_Wire_35_0.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_35_0.checkState(), "Wire_35_0")
        )
        w.cb_Wire_35_1.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_35_1.checkState(), "Wire_35_1")
        )
        w.cb_Wire_35_2.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_35_2.checkState(), "Wire_35_2")
        )
        w.cb_Wire_35_3.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_35_3.checkState(), "Wire_35_3")
        )
        w.cb_Wire_35_4.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_35_4.checkState(), "Wire_35_4")
        )
        w.cb_Wire_35_5.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_35_5.checkState(), "Wire_35_5")
        )
        w.cb_Wire_35_6.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_35_6.checkState(), "Wire_35_6")
        )
        w.cb_Wire_35_7.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_35_7.checkState(), "Wire_35_7")
        )
        w.cb_Wire_35_8.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_35_8.checkState(), "Wire_35_8")
        )
        w.cb_Wire_35_9.stateChanged.connect(
            lambda: w.checkBoxChangeAction(w.cb_Wire_35_9.checkState(), "Wire_35_9")
        )

        layout_child.addWidget(w.cb_Wire_35_0, 0, 0)
        layout_child.addWidget(w.cb_Wire_35_1, 1, 0)
        layout_child.addWidget(w.cb_Wire_35_2, 2, 0)
        layout_child.addWidget(w.cb_Wire_35_3, 3, 0)
        layout_child.addWidget(w.cb_Wire_35_4, 4, 0)
        layout_child.addWidget(w.cb_Wire_35_5, 5, 0)
        layout_child.addWidget(w.cb_Wire_35_6, 6, 0)
        layout_child.addWidget(w.cb_Wire_35_7, 7, 0)
        layout_child.addWidget(w.cb_Wire_35_8, 8, 0)
        layout_child.addWidget(w.cb_Wire_35_9, 9, 0)

        layout.addLayout(layout_child)
        layout.addLayout(commonpart_gen(w))

    return layout



from __future__ import annotations

import matplotlib.pyplot as plt
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.backends.backend_qt5agg import NavigationToolbar2QT as NavigationToolbar
from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import (
    QHBoxLayout,
    QLabel,
    QLineEdit,
    QScrollArea,
    QTextEdit,
    QVBoxLayout,
    QWidget,
)


class ConfirmWindow(QWidget):
    def __init__(self, parent=None):
        super(QWidget, self).__init__(parent)
        self.parent = parent

        self.parent.confirm_init_common(self, width=800, height=500)

    def back_page(self):
        self.parent.back_to_test()

    def check_json(self):
        self.parent.confirm_json()

    def upload_to_db(self):
        self.parent.upload_to_db()

    def add_info(self, Form_layout, label_str, form_text):
        label = QLabel()
        label.setText(label_str)

        if label_str == "Comment :":
            inner = QTextEdit()
            inner.setText(form_text)
            inner.setReadOnly(True)
            inner.setFocusPolicy(Qt.NoFocus)
            inner.setStyleSheet("background-color : linen; color: black;")

            editor = QScrollArea()
            editor.setWidgetResizable(True)
            editor.setWidget(inner)

        else:
            editor = QLineEdit()
            editor.setText(form_text)
            editor.setReadOnly(True)
            editor.setFixedWidth(200)
            editor.setFocusPolicy(Qt.NoFocus)
            editor.setStyleSheet("background-color : linen; color: black;")
        #        editor.setStyleSheet("background-color : azure")

        Form_layout.addRow(label, editor)

    #################################################################
    def make_layout(self):
        if (
            self.parent.info_dict["componentType"] == "MODULE"
            or self.parent.info_dict["componentType"] == "practice"
        ):
            return self.layout_ModuleQC()
        return self.layout_ModuleQC()

    def layout_ModuleQC(self):
        HBox = QHBoxLayout()
        Form_layout = self.parent.confirm_layout_common(self)

        self.add_info(
            Form_layout,
            "Comment :",
            self.parent.testRun["results"]["comment"],
        )

        current = []
        voltage = []
        current_error = []
        for v, i, sigmaI in zip(
            self.parent.testRun["results"]["IV_ARRAY"]["voltage"],
            self.parent.testRun["results"]["IV_ARRAY"]["current"],
            self.parent.testRun["results"]["IV_ARRAY"]["sigma current"],
        ):
            current.append(i)
            voltage.append(v)
            current_error.append(sigmaI)

        current_unit = "Negative Current [uA]"
        voltage_unit = "Negative Voltage [V]"

        graph = Create_Canvas(self)
        graph.setFixedHeight(380)
        graph.setFixedWidth(400)
        graph.graph_plot(voltage, current, voltage_unit, current_unit, current_error)
        graph.canvas_show()

        HBox.addLayout(Form_layout)
        HBox.addWidget(graph)

        return HBox


class Create_Canvas(QWidget):
    def __init__(self, parent):
        super(QWidget, self).__init__(parent)
        self.parent = parent
        self.ax = None

        self.figure = plt.figure()
        self.canvas = FigureCanvas(self.figure)
        toolbar = NavigationToolbar(self.canvas, self)

        layout = QVBoxLayout()
        layout.addWidget(self.canvas)
        layout.addWidget(toolbar)
        self.setLayout(layout)

    def graph_plot(self, x_list, y_list, x_unit, y_unit, y_error_list):
        self.ax = self.figure.add_subplot(111)
        self.ax.scatter(x_list, y_list, c="black")
        self.ax.errorbar(
            x_list, y_list, yerr=y_error_list, capsize=3, ecolor="black", c="black"
        )
        self.ax.set_title("Sensor I-V")
        self.ax.set_xlabel(x_unit)
        self.ax.set_ylabel(y_unit)
        self.figure.subplots_adjust(bottom=0.15)
        self.figure.subplots_adjust(left=0.25)

    def canvas_show(self):
        self.canvas.draw()



from __future__ import annotations

from PyQt5.QtWidgets import QComboBox, QGridLayout, QLabel, QPushButton, QWidget


class BareRegistWindow(QWidget):
    def __init__(self, parent=None):
        super(QWidget, self).__init__()
        self.parent = parent

        layout = QGridLayout()
        list_baretype = [""]
        list_FE_chipversion = [""]
        list_FE_thickness = [""]
        list_sensortype = [""]

        for bare_type in self.parent.baremodule_doc["types"]:
            list_baretype.append(bare_type["name"])

        property_SENSORTYPE = [
            prop
            for prop in self.parent.baremodule_doc["properties"]
            if prop["code"] == "SENSOR_TYPE"
        ]
        self.codeTable_SENSORTYPE = property_SENSORTYPE[0]["codeTable"]

        property_FECHIPVERSION = [
            prop
            for prop in self.parent.baremodule_doc["properties"]
            if prop["code"] == "FECHIP_VERSION"
        ]
        self.codeTable_FECHIPVERSION = property_FECHIPVERSION[0]["codeTable"]

        property_FETHICKNESS = [
            prop
            for prop in self.parent.baremodule_doc["properties"]
            if prop["code"] == "THICKNESS"
        ]
        self.codeTable_FETHICKNESS = property_FETHICKNESS[0]["codeTable"]

        for sensortype in self.codeTable_SENSORTYPE:
            list_sensortype.append(sensortype["value"])

        for fechipversion in self.codeTable_FECHIPVERSION:
            list_FE_chipversion.append(fechipversion["value"])

        for fethickness in self.codeTable_FETHICKNESS:
            list_FE_thickness.append(fethickness["value"])

        self.cb_baretype = QComboBox()
        self.cb_baretype.addItems(list_baretype)
        self.cb_FE_chipversion = QComboBox()
        self.cb_FE_chipversion.addItems(list_FE_chipversion)
        self.cb_FE_thickness = QComboBox()
        self.cb_FE_thickness.addItems(list_FE_thickness)
        self.cb_sensortype = QComboBox()
        self.cb_sensortype.addItems(list_sensortype)

        self.cb_baretype.activated[str].connect(self.onActivated_baretype)
        self.cb_FE_chipversion.activated[str].connect(self.onActivated_FE_chipversion)
        self.cb_FE_thickness.activated[str].connect(self.onActivated_FE_thickness)
        self.cb_sensortype.activated[str].connect(self.onActivated_sensortype)

        label_title = QLabel()
        label_title.setText("Select Bare Module Information")
        label_baretype = QLabel()
        label_baretype.setText("Bare Module Type")
        label_FE_chipversion = QLabel()
        label_FE_chipversion.setText("FE chip version")
        label_FE_thickness = QLabel()
        label_FE_thickness.setText("FE chip thickness")
        label_sensortype = QLabel()
        label_sensortype.setText("Sensor Type")

        button_next = QPushButton("&Register")
        button_next.clicked.connect(self.next_page)

        button_back = QPushButton("&Back")
        button_back.clicked.connect(self.back_page)

        layout.addWidget(label_title, 0, 0)
        layout.addWidget(label_baretype, 1, 0)
        layout.addWidget(self.cb_baretype, 1, 1)
        layout.addWidget(label_FE_chipversion, 2, 0)
        layout.addWidget(self.cb_FE_chipversion, 2, 1)
        layout.addWidget(label_FE_thickness, 3, 0)
        layout.addWidget(self.cb_FE_thickness, 3, 1)
        layout.addWidget(label_sensortype, 4, 0)
        layout.addWidget(self.cb_sensortype, 4, 1)
        layout.addWidget(button_next, 5, 1)
        layout.addWidget(button_back, 5, 0)
        self.setLayout(layout)

    def next_page(self):
        self.parent.register_baremodule()
        self.parent.feinfo()

    def back_page(self):
        self.parent.choose_menu()

    def onActivated_baretype(self, text):
        for bare_type in self.parent.baremodule_doc["types"]:
            if bare_type["name"] == text:
                self.parent.baremodule_info["type"] = bare_type["code"]

    def onActivated_FE_chipversion(self, text):
        for fechipversion in self.codeTable_FECHIPVERSION:
            if fechipversion["value"] == text:
                self.parent.baremodule_info["properties"][
                    "FECHIP_VERSION"
                ] = fechipversion["code"]

    def onActivated_FE_thickness(self, text):
        for fethickness in self.codeTable_FETHICKNESS:
            if fethickness["value"] == text:
                self.parent.baremodule_info["properties"]["THICKNESS"] = fethickness[
                    "code"
                ]

    def onActivated_sensortype(self, text):
        for sensortype in self.codeTable_SENSORTYPE:
            if sensortype["value"] == text:
                self.parent.baremodule_info["properties"]["SENSOR_TYPE"] = sensortype[
                    "code"
                ]



from __future__ import annotations

import logging

from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import (
    QGridLayout,
    QHBoxLayout,
    QLabel,
    QLineEdit,
    QPlainTextEdit,
    QPushButton,
    QScrollArea,
    QTextEdit,
    QVBoxLayout,
    QWidget,
)

log = logging.getLogger(__name__)


class NotSupportedError(Exception):
    pass


class ConfirmWindow(QWidget):
    def __init__(self, parent=None):
        super(QWidget, self).__init__(parent)
        self.parent = parent
        self.resize(400, 600)

        layout = QVBoxLayout()

        label_text = QLabel()
        label_text.setText('<center><font size="6"> Submission Preview</font></center>')
        layout.addWidget(label_text)

        grid_layout = self.make_infotable()
        hbox = QHBoxLayout()
        hbox.addLayout(grid_layout)

        layout.addLayout(hbox)
        layout.addStretch()

        layout_buttons = QHBoxLayout()
        button_back = QPushButton("Back")
        button_back.clicked.connect(self.parent.back_to_test)

        button_check = QPushButton("Check JSON")
        button_check.clicked.connect(self.parent.confirm_json)

        button_write = QPushButton("Register to LocalDB")
        button_write.clicked.connect(self.parent.upload_to_db)

        layout_buttons.addWidget(button_back)
        layout.addStretch()
        layout_buttons.addWidget(button_check)
        layout.addStretch()
        layout_buttons.addWidget(button_write)

        layout.addLayout(layout_buttons)

        layout.setSpacing(5)
        layout.setContentsMargins(20, 20, 20, 20)

        self.setLayout(layout)

    def get_maximum(self, contents_dict):
        N_char = []
        for key in contents_dict:
            label = QLabel()
            label.setText(contents_dict[key])
            N_char.append(label.sizeHint().width())
        return max(N_char)

    def make_infotable(self):
        log.info(self.parent.info_dict)
        contents = {
            "ATLAS Serial Number:": self.parent.info_dict["component"],
            "Component Type:": self.parent.info_dict["componentType"],
            "Current Stage:": self.parent.info_dict["currentStage"],
            "Test:": self.parent.info_dict["testType"],
        }

        if "front_defects" in self.parent.testRun["results"]["metadata"]:
            for tile, defect in self.parent.testRun["results"]["metadata"][
                "front_defects"
            ].items():
                if len(defect) == 0:
                    continue

                contents.update({f"Defects Tile-{tile}": "\n".join(defect)})

        if "front_comments" in self.parent.testRun["results"]["metadata"]:
            for tile, comment in self.parent.testRun["results"]["metadata"][
                "front_comments"
            ].items():
                if len(comment) == 0:
                    continue

                contents.update({f"Comment Tile-{tile}": comment})

        form_size = 600

        grid_layout = QGridLayout()
        self.add_infotable(
            grid_layout,
            "ATLAS Serial Number:",
            self.parent.info_dict["component"],
            form_size,
            0,
        )
        self.add_infotable(
            grid_layout,
            "Component Type:",
            self.parent.info_dict["componentType"],
            form_size,
            1,
        )
        self.add_infotable(
            grid_layout,
            "Current Stage:",
            self.parent.info_dict["currentStage"],
            form_size,
            2,
        )

        nRow = 3

        # --------------------------------------------------------------------

        if "front_defects" in self.parent.testRun["results"]["metadata"]:
            for tile, defect in self.parent.testRun["results"]["metadata"][
                "front_defects"
            ].items():
                if len(defect) == 0:
                    continue

                self.add_infotable(
                    grid_layout,
                    f"Front-side Defects Tile-{tile}:",
                    ",".join(defect),
                    form_size,
                    nRow,
                )
                nRow = nRow + 1

        for tile, comment in self.parent.testRun["results"]["metadata"][
            "front_comments"
        ].items():
            if len(comment) == 0:
                continue

            self.add_infotable(
                grid_layout,
                f"Front-side Comments Tile-{tile}:",
                comment,
                form_size,
                nRow,
            )
            nRow = nRow + 1

        if "back_defects" not in self.parent.testRun["results"]["metadata"]:
            return grid_layout

        # --------------------------------------------------------------------

        for tile, defect in self.parent.testRun["results"]["metadata"][
            "back_defects"
        ].items():
            if len(defect) == 0:
                continue

            self.add_infotable(
                grid_layout,
                f"Back-side Defects Tile-{tile}:",
                ",".join(defect),
                form_size,
                nRow,
            )
            nRow = nRow + 1

        for tile, comment in self.parent.testRun["results"]["metadata"][
            "back_comments"
        ].items():
            if len(comment) == 0:
                continue

            self.add_infotable(
                grid_layout,
                f"Back-side Comments Tile-{tile}:",
                comment,
                form_size,
                nRow,
            )
            nRow = nRow + 1

        return grid_layout

    def add_infotable(self, grid_layout, label_str, info_str, form_size, i):
        label_text = QLabel()
        label_text.setText('<font size="4">' + label_str + "</font>")

        if label_str.find("Comments") == 0:
            info_text = QPlainTextEdit()
            info_text.setPlainText(info_str)
            info_text.setFixedHeight(100)
        else:
            info_text = QLineEdit()
            info_text.setText(info_str)
            info_text.setFixedHeight(label_text.sizeHint().height() + 3)
        info_text.setReadOnly(True)
        info_text.setFocusPolicy(Qt.NoFocus)
        info_text.setStyleSheet("background-color : linen; color: black;")
        info_text.setFixedWidth(form_size)

        grid_layout.setSpacing(3)
        grid_layout.addWidget(label_text, i, 0)
        grid_layout.addWidget(info_text, i, 1)

    def label_gen(self, item):
        label = QLabel()
        label.setText(str(item))
        return label

    def add_info(self, Form_layout, label_str, form_text):
        label = QLabel()
        label.setText(label_str)

        if label_str == "Comment :":
            inner = QTextEdit()
            inner.setText(form_text)
            inner.setReadOnly(True)
            inner.setFocusPolicy(Qt.NoFocus)
            inner.setStyleSheet("background-color : linen; color: black;")

            editor = QScrollArea()
            editor.setWidgetResizable(True)
            editor.setWidget(inner)

        else:
            editor = QLineEdit()
            editor.setText(form_text)
            editor.setReadOnly(True)
            editor.setFocusPolicy(Qt.NoFocus)
            editor.setStyleSheet("background-color : linen; color: black;")
        #        editor.setStyleSheet("background-color : azure")

        Form_layout.addRow(label, editor)



from __future__ import annotations

import logging

from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import (
    QGridLayout,
    QHBoxLayout,
    QLabel,
    QLineEdit,
    QPlainTextEdit,
    QPushButton,
    QVBoxLayout,
    QWidget,
)

# from PyQt5.QtWidgets import QLabel, QPushButton, QVBoxLayout, QWidget
# from PyQt5.QtGUI import *
#

log = logging.getLogger(__name__)


class SummaryWindow(QWidget):
    def __init__(self, parent=None):
        super(QWidget, self).__init__(parent)
        self.parent = parent
        self.resize(400, 600)

        layout = QVBoxLayout()

        label_text = QLabel()
        label_text.setText('<center><font size="6"> Inspection Summary</font></center>')
        layout.addWidget(label_text)

        grid_layout = self.make_infotable()
        hbox = QHBoxLayout()
        hbox.addLayout(grid_layout)

        layout.addLayout(hbox)
        layout.addStretch()

        layout_buttons = QHBoxLayout()
        button_back = QPushButton("Back")
        button_back.clicked.connect(self.parent.inspection)

        button_write = QPushButton("Checkout Inspection")
        button_write.clicked.connect(self.write_data)

        layout_buttons.addWidget(button_back)
        layout.addStretch()
        layout_buttons.addWidget(button_write)

        layout.addLayout(layout_buttons)

        layout.setSpacing(5)
        layout.setContentsMargins(20, 20, 20, 20)

        self.setLayout(layout)

    def get_maximum(self, contents_dict):
        N_char = []
        for key in contents_dict:
            label = QLabel()
            label.setText(contents_dict[key])
            N_char.append(label.sizeHint().width())
        return max(N_char)

    def make_infotable(self):
        log.info(self.parent.parent.info_dict)
        contents = {
            "ATLAS Serial Number:": self.parent.parent.info_dict["component"],
            "Component Type:": self.parent.parent.info_dict["componentType"],
            "Current Stage:": self.parent.parent.info_dict["currentStage"],
        }

        for tile, anomaly in self.parent.anomaly_dic.items():
            if len(anomaly) == 0:
                continue

            contents.update({f"Defects Tile-{tile}": "\n".join(anomaly)})

        for tile, comment in self.parent.comment_dic.items():
            if len(comment) == 0:
                continue

            contents.update({f"Comment Tile-{tile}": comment})

        form_size = 600

        grid_layout = QGridLayout()
        self.add_infotable(
            grid_layout,
            "ATLAS Serial Number:",
            self.parent.parent.info_dict["component"],
            form_size,
            0,
        )
        self.add_infotable(
            grid_layout,
            "Component Type:",
            self.parent.parent.info_dict["componentType"],
            form_size,
            1,
        )
        self.add_infotable(
            grid_layout,
            "Current Stage:",
            self.parent.parent.info_dict["currentStage"],
            form_size,
            2,
        )

        nRow = 3

        for tile, anomaly in self.parent.anomaly_dic.items():
            if len(anomaly) == 0:
                continue

            self.add_infotable(
                grid_layout,
                f"Defects Tile-{tile}:",
                ",".join(anomaly),
                form_size,
                nRow,
            )
            nRow = nRow + 1

        for tile, comment in self.parent.comment_dic.items():
            if len(comment) == 0:
                continue

            self.add_infotable(
                grid_layout,
                f"Comments Tile-{tile}:",
                comment,
                form_size,
                nRow,
            )
            nRow = nRow + 1

        return grid_layout

    def add_infotable(self, grid_layout, label_str, info_str, form_size, i):
        label_text = QLabel()
        label_text.setText('<font size="4">' + label_str + "</font>")

        if label_str.find("Comments") == 0:
            info_text = QPlainTextEdit()
            info_text.setPlainText(info_str)
            info_text.setFixedHeight(100)
        else:
            info_text = QLineEdit()
            info_text.setText(info_str)
            info_text.setFixedHeight(label_text.sizeHint().height() + 3)
        info_text.setReadOnly(True)
        info_text.setFocusPolicy(Qt.NoFocus)
        info_text.setStyleSheet("background-color : linen; color: black;;")
        info_text.setFixedWidth(form_size)

        grid_layout.setSpacing(3)
        grid_layout.addWidget(label_text, i, 0)
        grid_layout.addWidget(info_text, i, 1)

    def label_gen(self, item):
        label = QLabel()
        label.setText(str(item))
        return label

    def write_data(self):
        self.parent.make_result()



from __future__ import annotations

import logging
import pprint

from PyQt5.QtCore import Qt
from PyQt5.QtGui import QPixmap
from PyQt5.QtWidgets import (
    QDialog,
    QGridLayout,
    QLabel,
    QLineEdit,
    QMessageBox,
    QPushButton,
    QRadioButton,
    QWidget,
)

log = logging.getLogger(__name__)


class FEInfoWindow(QWidget):
    def __init__(self, parent=None):
        super(QWidget, self).__init__()
        self.parent = parent

        self.layout = QGridLayout()

        self.FE_CHIP_COMMON = "20UPGFC"
        self.inputtype = "NULL"

        self.pp = pprint.PrettyPrinter(depth=1)
        label_title = QLabel()
        label_title.setText("Input FE chips Information")
        label_exp = QLabel()
        label_exp.setText(
            "input as ATLAS Serial Number or hex 0xBWWXY, B: batch number, WW: wafer number in batch, X: row, Y: col"
        )
        self.rb_hex = QRadioButton("Hex")
        self.rb_atlsn = QRadioButton("ATLAS Serial Number")

        self.rb_hex.toggled.connect(self.onClicked)
        self.rb_atlsn.toggled.connect(self.onClicked)

        label_chip1 = QLabel()
        label_chip1.setText("chip1:")
        label_chip2 = QLabel()
        label_chip2.setText("chip2:")
        label_chip3 = QLabel()
        label_chip3.setText("chip3:")
        label_chip4 = QLabel()
        label_chip4.setText("chip4:")

        self.edit_chip1 = QLineEdit()
        self.edit_chip2 = QLineEdit()
        self.edit_chip3 = QLineEdit()
        self.edit_chip4 = QLineEdit()

        button_quad = QPushButton("&quad reference")
        button_quad.clicked.connect(self.show_reference)

        button_next = QPushButton("&Assemble")
        button_next.clicked.connect(self.next_page)
        button_skip = QPushButton("&Skip")
        button_skip.clicked.connect(self.skip_page)
        button_back = QPushButton("&Back")
        button_back.clicked.connect(self.back_page)

        # self.img = QLabel(self)
        # pixmap = QPixmap("img/baremodule.jpg")
        # self.img.setPixmap(pixmap)

        self.layout.addWidget(label_title, 0, 0)
        self.layout.addWidget(label_exp, 1, 0, 1, 3)
        self.layout.addWidget(self.rb_hex, 2, 0)
        self.layout.addWidget(self.rb_atlsn, 3, 0)

        self.layout.addWidget(button_quad, 4, 2)

        self.layout.addWidget(label_chip1, 5, 0)
        self.layout.addWidget(self.edit_chip1, 5, 1)
        self.layout.addWidget(label_chip2, 6, 0)
        self.layout.addWidget(self.edit_chip2, 6, 1)
        self.layout.addWidget(label_chip3, 7, 0)
        self.layout.addWidget(self.edit_chip3, 7, 1)
        self.layout.addWidget(label_chip4, 8, 0)
        self.layout.addWidget(self.edit_chip4, 8, 1)

        self.layout.addWidget(button_next, 9, 2)
        self.layout.addWidget(button_skip, 9, 1)
        self.layout.addWidget(button_back, 9, 0)

        self.setLayout(self.layout)

    def show_reference(self):
        self.parent.refwindow = ReferenceWindow(self.parent)
        self.parent.refwindow.show()
        # self.layout.addWidget(self.img,5,2,4,1)

    def onClicked(self):
        radiobtn = self.sender()
        if radiobtn.isChecked():
            if radiobtn.text() == "Hex":
                self.inputtype = "hex"
            elif radiobtn.text() == "ATLAS Serial Number":
                self.inputtype = "atlsn"

    def next_page(self):
        if self.inputtype == "hex":
            try:
                chip1_decimal = int(self.edit_chip1.text(), 16)
                chip2_decimal = int(self.edit_chip2.text(), 16)
                chip3_decimal = int(self.edit_chip3.text(), 16)
                chip4_decimal = int(self.edit_chip4.text(), 16)

                str_chip1_decimal = self.add_zero(str(chip1_decimal))
                str_chip2_decimal = self.add_zero(str(chip2_decimal))
                str_chip3_decimal = self.add_zero(str(chip3_decimal))
                str_chip4_decimal = self.add_zero(str(chip4_decimal))

                chip1_str = self.FE_CHIP_COMMON + str_chip1_decimal
                chip2_str = self.FE_CHIP_COMMON + str_chip2_decimal
                chip3_str = self.FE_CHIP_COMMON + str_chip3_decimal
                chip4_str = self.FE_CHIP_COMMON + str_chip4_decimal

                self.parent.baremodule_info["child"]["FE_CHIP"].append(chip1_str)
                self.parent.baremodule_info["child"]["FE_CHIP"].append(chip2_str)
                self.parent.baremodule_info["child"]["FE_CHIP"].append(chip3_str)
                self.parent.baremodule_info["child"]["FE_CHIP"].append(chip4_str)
            except Exception:
                log.exception("no FE chips info!!")
        elif self.inputtype == "atlsn":
            self.check_and_assemble(self.edit_chip1.text())
            self.check_and_assemble(self.edit_chip2.text())
            self.check_and_assemble(self.edit_chip3.text())
            self.check_and_assemble(self.edit_chip4.text())
        else:
            QMessageBox.warning(
                None, "Warning", "Choose input style option", QMessageBox.Ok
            )

        self.parent.assemble_fechip()
        self.parent.sensorinfo()

        # self.parent.connectITKPD()

    def check_and_assemble(self, fe_serial):
        if fe_serial != "":
            try:
                doc = self.parent.get_componentinfo(fe_serial)
            except Exception:
                QMessageBox.warning(
                    None, "Error", "Cannot get FE chip information", QMessageBox.Ok
                )
                return 1
            parents_list = doc["parents"]
            isAssembled = False
            if parents_list is not None:
                for parent in parents_list:
                    if (
                        parent["componentType"]["code"] == "BARE_MODULE"
                        and parent["history"][-1]["action"] == "assembly"
                    ):
                        isAssembled = True
                        QMessageBox.critical(
                            None,
                            "Error",
                            fe_serial + " is already assembled.",
                            QMessageBox.Ok,
                        )
                return None

            if not isAssembled:
                self.parent.baremodule_info["child"]["FE_CHIP"].append(fe_serial)
                return None
            return None
        return None

    def back_page(self):
        self.parent.bareregist()

    def skip_page(self):
        self.parent.sensorinfo()

    def add_zero(self, chip_str):
        while len(chip_str) < 7:
            chip_str = "0" + chip_str
        return chip_str


class ReferenceWindow(QWidget):
    def __init__(self, parent=None):
        super(QWidget, self).__init__(parent)
        self.refbox = QDialog(parent)
        self.parent = parent

        layout = QGridLayout()

        self.img = QLabel(self)
        # pixmap = QPixmap("img/baremodule.jpg")
        pixmap = QPixmap("img/assembledmodule.jpg")
        self.img.setPixmap(pixmap)

        layout.addWidget(self.img, 0, 0)
        self.refbox.setLayout(layout)

    def show(self):
        self.setWindowModality(Qt.NonModal)
        self.refbox.show()



from __future__ import annotations

import logging
import sys

import itkdb
from PyQt5.QtWidgets import QApplication, QMainWindow

from module_qc_nonelec_gui.GUI.registration_bare_module import (
    bareinfo_win,
    bareregist_win,
    connectpd_win,
    feinfo_win,
    initial_win,
    sensorinfo_win,
)

log = logging.getLogger(__name__)


class MainWindow(QMainWindow):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.parent = parent
        self.setWindowTitle("Bare module registration tool")
        self.setGeometry(0, 0, 340, 255)

        # self.pd_client = self.parent.pd_client
        # self.connectITKPD()
        self.baremodule_info = {
            "project": "P",
            "subproject": "PG",
            "institution": "",
            "componentType": "BARE_MODULE",
            "type": "",
            "properties": {"FECHIP_VERSION": "", "THICKNESS": "", "SENSOR_TYPE": ""},
            "child": {"FE_CHIP": [], "SENSOR_TILE": []},
        }

        self.baremodule_doc = {}
        self.fechip_doc = {}
        self.sensor_doc = {}
        self.user_info = {}
        self.connectpd()

        # other member variables
        self.bare_doc = None
        self.initial_wid = None
        self.bareinfo_wid = None
        self.bareregist_wid = None
        self.feinfo_wid = None
        self.sensorinfo_wid = None
        self.connectpd_wid = None
        self.u = None

    def init_ui(self):
        self.choose_menu()

    def choose_menu(self):
        self.bare_doc = {}
        self.baremodule_info["child"]["FE_CHIP"] = []
        self.baremodule_info["child"]["SENSOR_TILE"] = []
        self.initial_wid = initial_win.InitialWindow(self)
        self.parent.update_widget(self.initial_wid)
        log.info("\n---------------------------------------------------------------")

    def bareupdate(self):
        self.bareinfo_wid = bareinfo_win.BareInfoWindow(self)
        self.parent.update_widget(self.bareinfo_wid)

    def bareregist(self):
        self.bareregist_wid = bareregist_win.BareRegistWindow(self)
        self.parent.update_widget(self.bareregist_wid)

    def set_windowsize(self, x, y):
        self.setGeometry(0, 0, x, y)

    def feinfo(self):
        self.feinfo_wid = feinfo_win.FEInfoWindow(self)
        self.parent.update_widget(self.feinfo_wid)

    def sensorinfo(self):
        self.sensorinfo_wid = sensorinfo_win.SensorInfoWindow(self)
        self.parent.update_widget(self.sensorinfo_wid)

    def connectITKPD(self):
        self.connectpd_wid = connectpd_win.ConnectPDWindow(self)
        self.parent.update_widget(self.connectpd_wid)

    def scale_window(self, x, y):
        QApplication.processEvents()
        self.set_windowsize(x, y)

    def connectpd(self):
        # def connectpd(self,code1,code2):
        #     token = self.process_request(code1,code2)
        #     if token == 0:
        #         log.info('fail to connect ITk PD')

        # else:
        #     log.info('success to connect ITk PD')
        # self.userinfo = self.parent.pd_client.get("getUser", json={ "userIdentity":self.parent.u.identity})
        self.baremodule_info["institution"] = self.parent.userinfo["institutions"][0][
            "code"
        ]
        self.baremodule_doc = self.parent.pd_client.get(
            "getComponentTypeByCode", json={"project": "P", "code": "BARE_MODULE"}
        )
        # log.info("-----------------------")
        self.fechip_doc = self.parent.pd_client.get(
            "getComponentTypeByCode", json={"project": "P", "code": "FE_CHIP"}
        )
        # log.info(self.fechip_doc)
        # log.info("-----------------------")
        self.sensor_doc = self.parent.pd_client.get(
            "getComponentTypeByCode", json={"project": "P", "code": "SENSOR_TILE"}
        )
        # log.info(self.sensor_doc)
        # self.choose_menu()

    def register_baremodule(self):
        #####################
        #       doc = self.parent.pd_client.get('getComponent', json={"component": "20UPGB40500017"})
        #####################
        bare_ins = self.parent.pd_client.post(
            "registerComponent", json=self.baremodule_info
        )
        log.info("Register bare module!!")
        self.bare_doc = self.parent.pd_client.get(
            "getComponent", json={"component": bare_ins["component"]["serialNumber"]}
        )
        log.info("-----------------------------------")
        log.info("### Registered components ###")
        log.info("Bare Module: " + bare_ins["component"]["serialNumber"])
        log.info("")
        log.info("Check the components from your component page.")
        log.info("Link: https://itkpd-test.unicorncollege.cz/myComponents")
        # msgBox = QMessageBox.information(None, 'Information',bare_ins["component"]["serialNumber"] + " is registered", QMessageBox.Ok)
        # if msgBox == QMessageBox.Ok:
        #     self.feinfo()
        # msgBox = QMessageBox()
        # msgBox.setText("bare module is registered")
        # msgBox.setInformativeText(bare_ins["component"]["serialNumber"])
        # msgBox.setStandardButtons(QMessageBox.Ok)
        # ret = msgBox.exec_()
        # if ret == QMessageBox.Ok:

    def get_bareinfo(self, atlsn):
        self.bare_doc = self.parent.pd_client.get(
            "getComponent", json={"component": atlsn}
        )

    def get_componentinfo(self, atlsn):
        return self.parent.pd_client.get("getComponent", json={"component": atlsn})

    def assemble_fechip(self):
        for j in range(4):
            json = {
                "parent": self.bare_doc["code"],
                "slot": self.bare_doc["children"][j]["id"],
                "child": self.baremodule_info["child"]["FE_CHIP"][j],
            }
            self.parent.pd_client.post("assembleComponentBySlot", json=json)
        log.info("Assemble FE chips!!")

    def assemble_sensor(self):
        json = {"parent": self.bare_doc["code"], "child": self.sensor_id}
        self.parent.pd_client.post("assembleComponent", json=json)
        log.info("Assemble SENSOR TILE!!")

    def update_baremodule(self):
        if "FE_CHIP" in self.baremodule_info["child"]:
            for _j in range(4):
                json = {
                    "parent": self.bare_doc["code"],
                }

        if "SENSOR_TILE" in self.baremodule_info["child"]:
            json = {"parent": self.bare_doc["code"], "child": self.sensor_id}
            self.parent.pd_client.post("assembleComponent", json=json)

    def process_request(self, code1, code2):
        try:
            self.u = itkdb.core.User(access_code1=code1, access_code2=code2)
            self.u.authenticate()
            self.parent.pd_client = itkdb.Client(user=self.u)
            log.info("Authorized.")
            request = 1
        except Exception:
            log.exception(
                "Not authorized. Please login for ITkPD by using itkpd-interface/authenticate.sh"
            )
            request = 0
        return request

    #    def update_widget(self, w):
    #        self.setCentralWidget(w)
    #        self.show()
    #        self.scale_window(360,270)

    def close_window(self):
        self.close()

    def call_another_window(self, window):
        self.hide()
        self.update_statusbar(window)
        window.init_ui()

    def back_to_last_window(self, window):
        self.hide()
        window.show()

    def receive_return(self, window):
        window.hide()
        self.show()


if __name__ == "__main__":
    app = QApplication(sys.argv)
    main_window = MainWindow()
    main_window.show()
    sys.exit(app.exec_())
